pro implement_rrg_pixelscale

;Analyses images for varying pixelscale vs. n_effective and plots results
;Transfer to /simage directory to compile and execute.

plate_scale = fltarr(15)
n_eff = fltarr(15)

for i = 1, 15, 1 do begin
print, i
pixel_scale = (0.025+(0.025*i))
suffix = strcompress(pixel_scale,/rem)

star_file = simage_shapelets_paths(1,/silent) + "star_exp400s_pixelscale_"+strcompress(pixel_scale,/rem)
rrg_measure_moms,star_file+'_noisy.fits', star_file+'_noisy.cat', star_file+'_noisy.momstar', min_rad=2,mult=2, /stellar

galaxy_file = simage_shapelets_paths(1,/silent) + "galaxy_exp400s_pixelscale_"+strcompress(pixel_scale,/rem)
rrg_measure_moms,galaxy_file+'_noisy.fits', galaxy_file+'_noisy.cat', galaxy_file+'_noisy.moms', min_rad=2,mult=2

restore, star_file+'_noisy.momstar'
data = histogram(moms.xx, locations = values, nbins = 100, min = 0.0)
peak = values(where(data eq max(data)))
min_value = peak/1.2
max_value = peak*1.2

sel=where((moms.xx gt min_value(0))and(moms.xx lt max_value(0))and(moms.yy gt min_value(0))and(moms.yy lt max_value(0)))


medianxx =  median(moms.xx(sel))
medianyy =  median(moms.yy(sel))
d_psf = sqrt(0.5*(medianxx+medianyy))

restore, galaxy_file+'_noisy.moms'
d=sqrt(0.5*(moms.xx+moms.yy))
sel=where((moms.xx gt 0)and(moms.yy gt 0)and (moms.sn gt 10)and(d gt 1.25*d_psf))

plate_scale(i-1) = pixel_scale
n_eff(i-1) = n_elements(sel)/6.0

endfor

set_plot, 'ps'
Device,  filename='n_eff_vs_pixelscale.eps', /encapsulated, xsize=6.5, ysize=6., /inches

plot, plate_scale, n_eff, xtitle='Plate-scale  !N[arcseconds/pixel]', ytitle ='n!Leff   !N[ S/N > 10, d > 1.25*d!Lpsf!N ]', charsize=1.6, charthick = 3.5, yrange=[0,30], thick=4.0, xrange=[0.0,0.45]

Device, /close_file


stop
end


