;$Id: shapelets_create_shapecat.pro, v1$
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_CREATE_SHAPECAT
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Initiliase a band new shapelet catalogue structure.
;      Works like an IDL __DEFINE procedure, but for an anonymous structure
;      (which we need because they will shrink or expand to accomodate
;      varying amounts of data).
;
; INPUTS:
;      
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      POLAR  - Whether or not the expansion should be in polar (set) or
;               Cartesian (unset) shapelet coefficients.
;
; OUTPUTS:
;      SHAPECAT - Returns the new shapecat structure.
;
; MODIFICATION HISTORY:
;      Jul 05 - Written by Richard Massey.
;-

function simage_shapelets_create_shapecat, N_MAX,                          $
                                    NAME=name,                      $
        			    COMMENT=comment,                $
        			    CHISQ_TARGET=chisq_target,      $
        			    CHISQ_TOLERANCE=chisq_tolerance,$
        			    CHISQ_FLATNESS=chisq_flatness,  $
        			    BETA_TOLERANCE=beta_tolerance,  $
				    THETA_MIN_GEOM=theta_min_geom

COMPILE_OPT idl2

; Parse input
if not keyword_set(name)    then name=""
if not keyword_set(comment) then comment=""

; Initialise history
systime=systime()
date=strmid(systime,0,4)+strmid(systime,8,3)+strmid(systime,4,4)+strmid(systime,20,4)
time=strmid(systime,11,5)
history="Created at "+time+" on "+date+" "+comment+"."

; Set default numerical values
if n_elements(chisq_target) eq 0    then chisq_target_local=1.0    else chisq_target_local = chisq_target      
if n_elements(chisq_tolerance) eq 0 then chisq_tolerance_local=1.0 else chisq_tolerance_local = chisq_tolerance
if n_elements(chisq_flatness) eq 0  then chisq_flatness_local=0.01 else chisq_flatness_local = chisq_flatness  
if n_elements(beta_tolerance) eq 0  then beta_tolerance_local=0.03 else beta_tolerance_local = beta_tolerance  
if n_elements(theta_min_geom) eq 0  then theta_min_geom_local=0.2  else theta_min_geom_local = theta_min_geom  

; List flag definitions
flag_interpret=["OK",							                         	                    $ ; 0
                "Bounced off geometrical constraints",                                           	                    $ ; 1
                "Converged by flatness limit - shapelets may incompletely represent object",     	                    $ ; 2
                "Chi^2 not monotonic in n_max - shapelets may incompletely represent object",                               $ ; 3
                "Amoeba dithered about and did not converge to target chi^2 - shapelets may incompletely represent object", $ ; 4
                "Focus iteration did not converge to target chisq - shapelets may incompletely represent object",           $ ; 5
                "Maximum n_max reached - shapelets may incompletely represent object",           	                    $ ; 6
                "Centroid wandered, pushing basis functions off the edge of the postage stamp",                             $ ; 7
                "Fatal crash during focus routine"]                                                                           ; 8
flag_interpret_mini=["OK","Bounced","Flatness","Worsening","Amoeba dithered","Focus didn't coverge","Max !8n!X!imax!n","Centroid wandered","Fatal crash"]

; Create actual structure
focus={name:name,            		      $
       type:"focus",         		      $
       history:history,      		      $
       beta:0.,              		      $
       n_max:0,              		      $
       x:fltarr(2),         		      $
       chisq:!values.f_infinity,              $
       chisq_target:chisq_target_local,       $
       chisq_tolerance:chisq_tolerance_local, $
       chisq_flatness:chisq_flatness_local,   $
       chisq_abs_target:chisq_target_local,   $
       beta_tolerance:beta_tolerance_local,   $
       theta_min_geom:theta_min_geom_local,   $
       n_max_guess:0,           	      $
       beta_guess:0.,           	      $
       x_guess:fltarr(2),      		      $
       n_iterations:0,          	      $
       beta_history:0.,         	      $
       n_max_history:0,         	      $
       x_history:fltarr(1,2),  		      $
       chisq_history:0.,        	      $
       flag:0,                  	      $
       flag_interpret:flag_interpret,         $
       flag_interpret_mini:flag_interpret_mini}

; Return the shiny new decomp structure
return,focus

end
