pro simage_shapelets_read_shapecat_hdf,shapecat,       $
                                FILTER=filter,  $
						  DECONV=deconv,  $
						  N_MAX=n_max,    $
                                POLAR=polar,    $
                                SILENT=silent,  $
                                _ref_extra = ex

;$Id: shapelets_read_shapecat_hdf.pro,v 1.1.1.1 2005/04/08 20:51:58 fwh Exp $
;
; Copyright ? 2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_READ_SHAPECAT_HDF
;
; PURPOSE:
;       Reads in and concatenates shapelet catalogues containing all of the
;       objects from both Hubble Deep Fields.
;
; INPUTS:
;       None.
;
; OPTIONAL INPUTS:
;       FILTER  - HDF filter e.g. 606 or 814. Default: 814
;       N_MAX   - Truncate catalogue at this shapelet parameter.
;
; KEYWORD PARAMETERS:
;       POLAR   - Convert to polar shapelet coefficents automatically on load,
;                 in reduced {r,theta} form (cf shapelets_polar_reduce.pro).
;       MOMENTS - Calculate object shape moments (astrometry, photometry etc)
;                 automatically on load.
;       DECONV  - Read PSF-deconvolved catalogue (if available)
;       PARITY  - Rotate all objects automatically on load so that their
;                 ellipticity is aligned with the x axis then flip, if
;                 necessary, so that all their isophotes begin to twist in a
;                 clockwise sense.
;       RESIZE  - Rescale so that all shapelets use the same beta.
;       SILENT  - Operates silently.
;
; OUTPUTS:
;       shapecat- Catalogue (shapecat structure) returned in memory.
;
; MODIFICATION HISTORY:
;       May 04 - IDL save file storage system implemented by RM.
;       Mar 04 - Split into _read and _concatenate_shapecats by RM.
;       Jan 02 - Written by Richard Massey.
;-

COMPILE_OPT idl2


; Set up filenames
if n_params() eq 0 then message,'USE: simage_shapelets_read_shapecat_hdf,shapecat !'
if keyword_set(filter) then filter2='f'+strtrim(filter,2) else filter2='f814'
if keyword_set(deconv) then filter3=filter2+"_deconv" else filter3=filter2
in_dir=simage_shapelets_paths(2,silent=silent)


; Read in catalogues
message,"Reading in shapelet catalogue of HDF north",/info,noprint=silent
simage_shapelets_read_shapecat,shapecat_n,'HDF-N_'+filter3,$
  moments=moments,polar=polar,/silent,n_max=n_max, _extra= ex
message,"Reading in shapelet catalogue of HDF south",/info,noprint=silent
simage_shapelets_read_shapecat,shapecat_s,'HDF-S_'+filter3,$
  moments=moments,polar=polar,/silent,n_max=n_max, _extra= ex


; Concatenate catalogues
shapecat=simage_shapelets_concatenate_shapecats(shapecat_n,shapecat_s)
shapecat.name="HDFs"     
message,'HDF shapelet catalogues successfully in memory',/info,noprint=silent


end
