pro simage_shapelets_reduce_nmax,decomp,tol=tol,plot=plot
;take a decomp structure and find the smallest n_max such that the
;error is smaller than a certain amount
;good for reducing the order which improves execution time
;tol specifies the error on the distrubution divided by the peak value
;the critrion is that the mean chi-square is less than 1

if n_params() eq 0 then begin
    print,'-syntax simage_shapelets_reduce_nmax,decomp'
    return
endif

if n_elements(tol) eq 0 then tol=0.0001
nmin=0
n_max=decomp.n_max
if n_max le nmin then begin
    print,'WARNING - n_max is already less than n_min',n_max
    return
endif

dec=decomp


simage_shapelets_recomp,decomp,recomp
s=size(recomp,/dim)
nx=s[0]
ny=s[1]

err2=(max(recomp)*tol)^2

for i=nmin, n_max do begin
    dec.n_max=i
    simage_shapelets_recomp,dec,rec
    chi2=mean((rec-recomp)^2/err2)
    if keyword_set(plot) then begin
        print,'n_max = ',i,'  Chi2 = ',chi2
        plot,recomp[*,ny/2],psym=-1
        oplot,rec[*,ny/2],psym=-7,color=!red
        
        ploterror,findgen(nx),rec[*,ny/2]-recomp[*,ny/2],sqrt(err2)+fltarr(nx)
        
        print,'ok?'
        key=get_kbrd(20)
        if key eq 'q' then return
    endif
    if chi2 lt 1.0 then begin
        ;decomp.n_max=i
        simage_shapelets_extend_nmax,decomp,i-n_max
        if keyword_set(plot) then print,'Final n_max = ',i
        return
    endif
endfor

return
end


