;$Id: smooth_gaussian.pro, v1.0$
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;
; ************************************************************************
; ************************************************************************
;
; NAME:
;      SMOOTH_GAUSSIAN
;
; CATEGORY:
;      Smoothing.
;
; PURPOSE:
;      Smooth an image with a Gaussian.
;
; INPUTS:
;      IMAGE - Image array to be smoothed.
;      FWHM  - FWHM of Gaussian used for smoothing [pixels].
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      IMAGE - Smoothed image array.
;
; KNOWN BUGS:
;      Doesn't conserve object centroid: translation occurs simultaneously with
;      the convolution .
;      Doesn't conserve object flux!
;
; MODIFICATION HISTORY:
;      Aug 04 - Written by Richard Massey
 
pro simage_smooth_gaussian, image, fwhm
 
COMPILE_OPT idl2
 
; Manufacture a Gaussian
simage_shapelets_read_psf,psf,fwhm,/SILENT
n_pixels=size(image,/dimensions)
psf.n_pixels=n_pixels
psf.x=n_pixels/2.
psf.x=[0,0];[0.5,0.5]
simage_shapelets_recomp,psf,gaussian
gaussian=gaussian+reverse(gaussian,1)
gaussian=gaussian+reverse(gaussian,2)
;
; BUT THIS KERNEL ISN'T NORMALISED TO UNITY!!!!!
message,'Warning: Kernel not properly normalised',/info
gaussian=gaussian/total(gaussian)
;
;
;gaussian=gaussian/total(gaussian)*4
 
; FT Gaussian
gaussian=complex(temporary(gaussian),fltarr(n_pixels))
gaussian=fft(gaussian,/overwrite)


; FT Image
totalimage=total(image)
image=complex(temporary(image),fltarr(n_pixels))
image=fft(image,/overwrite)
 
; Perform smoothing
image=temporary(image)*gaussian
 
; Inverse FT
image=float(fft(image,/inverse,/overwrite))
image=image*totalimage/total(image)
 
end
