package Simgen::Engine;

use strict;
use warnings;
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::IO;
use Simgen::Locale;

#
#
# Execute the desired software, as indicated in @toDo.
#
#
sub Run {

    my $method = shift;
    my $routine = shift;
    my $software = shift;
    my $a = shift;
    my $kishN = shift;

    if ( !scalar($$software{$method}) ) {
	
	message("Method '$method' unknown.");
	die;
    }
    if ( !scalar(%{$$software{$method}{$routine}}) ) {
	
	message("Routine '$routine' unkown for method '$method'.");
	die;
    }
    
    my $argVal = "";
    my $argList = "";
    
    for my $argName ( @{$$software{$method}{$routine}{args}} ) {
	
	$argVal = $$a{$argName};
	
	if ( $argVal eq "" ) {
	    
	    message("Arg $argVal doesn't appear in master arg list *or* wasn't initialized!");
	    die;
	}
	if ($argName eq "gamma") {
	    
	    if ($$argVal->[0] eq "" || $$argVal->[1] eq "" ) {

		message("Gamma not initialized properly.");
		die;
	    }

	    $argVal = "@$argVal[0] @$argVal[1]";
	}

	$argList = "$argList $argVal";
    }

    my $exec = $$software{$method}{$routine}{exec};

    if (Simgen::Global::dryRun) {
	message("Executing $exec $argList");
    } else {
	&wait4license();
	&run("$exec $argList");

# 	if ( Simgen::Global::Institute eq "PDSF" ) {
# 	    die "PDSF part not tested yet";
# 	    my $scriptFilename = "simsub.csh";
# 	    open(SCRIPT, "> $scriptFilename ") || die("can't open logfile:	$!");
# 	    print SCRIPT "#!/bin/csh\n";
# 	    print SCRIPT "$exec $argList\n";
# 	    print SCRIPT "\n" && close(SCRIPT);
# 	    &run("chmod +x $scriptFilename");
# 	    &run("touch ${scriptFilename}.bsub_out");
# 	    &run("bsub -q long -o ${scriptFilename}.bsub_out < $scriptFilename");
# 	    &run("rm $scriptFilename");
# 	    exit 0;
# 	} elsif ( Simgen::Global::Institute eq "Caltech" ) {
# 	    &wait4license();
# 	    &run("ssh -x kishar$kishN '$exec $argList'");
# 	} else {
# 	    message("Unable to run becuase institute is unknown.");
# 	}
    }
} # End sub Run.



#
#
# Wait for an available IDL license, and don't hog them.
#
#
sub wait4license {

    my $ready = false;
    my $IDL_DIR = Simgen::Locale::IDLdir;
    my $licensesTot = Simgen::Locale::Licenses("total");
    my $licenses4me = Simgen::Locale::Licenses("available");
    while (!$ready) {
	chomp(my $n_6  = int `$IDL_DIR/bin/lmstat -A | grep  '6 licenses' | wc -l`);
	chomp(my $n_10 = int `$IDL_DIR/bin/lmstat -A | grep '10 licenses' | wc -l`);
	my $inUse = 10*$n_10 + 6*$n_6;
	chomp($n_6  = int `$IDL_DIR/bin/lmstat -A | grep  '6 licenses' | grep fwh | wc -l`);
	chomp($n_10 = int `$IDL_DIR/bin/lmstat -A | grep '10 licenses' | grep fwh | wc -l`);
	my $inUseByMe = 10*$n_10 + 6*$n_6;
	
	message("$inUse out of $licensesTot IDL licenses in use, $inUseByMe by me.");

	if ($inUse >= $licensesTot*6.0/6.0 || $inUseByMe >= $licenses4me) {
	    chomp(my $datum = `date`);
	    message("IDL unavailable at ${datum}.");
	    sleep 60;
	    $ready = false;
	} else {
	    $ready = true;
	    message("IDL is free.");
	}
    }
}



sub MakeRT {

    my $algo = shift;
    my $algoMethod = shift;
    my $software = shift;
    my %software = %$software;

    my $exec = $software{'utils'}{'make_rt'}{'exec'};
    die "Can't find executable for $algo $algoMethod" unless $exec;
    my $dot_pro = $software{'utils'}{'make_rt'}{$algo}{$algoMethod};
    die "Can't find arguments to executable for $algo $algoMethod" unless $dot_pro;
    run(["$exec $dot_pro"],1);
}


END {}

1;
