package Simgen::Global;

use strict;
use warnings;
use Simgen::SuperGlobal;

BEGIN {

    use Exporter;
    our (@ISA, @EXPORT);
    
    
    @ISA = qw(Exporter);
    @EXPORT = qw(verbose makeLog dryRun Scheme);
    
}

sub verbose { 
    return true;
#    return false;	
};

sub makeLog { 
    return true;
#    return false;
};

sub dryRun { 
#    return true;
    return false;
};

sub want2do {
#    return "simages";
#    return "rrg";
    return "want2do is obsolete";
}

sub Institute { 
    
    my $val = "";

    my %KnownInstitutes = (
			   "Caltech" => [ "anshar",
					  "kishar1",
					  "kishar2",
					  "kishar3",
					  "kishar4",
					  "kishar5",
					  "kishar6",
					  "kishar7" ],
			   "PDSF" => [ "pc" ]
			   );

    chomp( my $host = `hostname` );
    foreach my $institute ( keys %KnownInstitutes ) {
	foreach my $machine ( @{$KnownInstitutes{$institute}} ) {
	    $host =~ s/pc..../pc/g;
	    $val = $institute if $machine eq $host;
	}
    }

    die "Host $host is not configured for simgen." unless $val;

    return $val;
} # End sub Insititute.

sub Scheme {

    my $scheme = shift;
    my $hashRef = "ERROR";

    my %possibleSchemes = (
			   old_snap_psf => {
			       pixel_scale => [ 0.04,
						0.05,
						0.06,
						0.07,
						0.08,
						0.09,
						0.10,
						0.11,
						0.12,
						0.13,
						0.14,
						0.15,
						0.16 ],
			       gamma => [ [  0.000, -0.050 ],
					  [  0.000, -0.025 ],
					  [  0.000,  0.000 ],
					  [  0.000,  0.025 ],
					  [  0.000,  0.050 ],
					  [ -0.050,  0.000 ],
					  [ -0.025,  0.000 ],
					  [  0.025,  0.000 ],
					  [  0.050,  0.000 ] ],
			       dither => [ [ 0.0, 0.0 ] ],
			       psf_file => "SNAP_PSF",
			       rrg_dir => "rrg01",
			   }, # End old_snap_psf scheme.

			   new_snap_psf => {
			       pixel_scale => [ 0.04,
						0.05,
						0.06,
						0.07,
						0.08,
						0.09,
						0.10,
						0.11,
						0.12,
						0.13,
						0.14,
						0.15,
						0.16 ],
			       gamma => [ [  0.000, -0.050 ],
					  [  0.000, -0.025 ],
					  [  0.000,  0.000 ],
					  [  0.000,  0.025 ],
					  [  0.000,  0.050 ],
					  [ -0.050,  0.000 ],
					  [ -0.025,  0.000 ],
					  [  0.025,  0.000 ],
					  [  0.050,  0.000 ] ],
			       dither => [ [ 0.0, 0.0 ] ],
			       psf_file => "fft11.fits",
			       rrg_dir => "rrg01",
			   }, # End new_snap_psf scheme.

			   dither_new_psf => {
			       pixel_scale => [ 0.04,
						0.05,
						0.06,
						0.07,
						0.08,
						0.09,
						0.10,
						0.11,
						0.12,
						0.13,
						0.14,
						0.15,
						0.16 ],
			       gamma => [ [  0.000, -0.050 ],
					  [  0.000, -0.025 ],
					  [  0.000,  0.000 ],
					  [  0.000,  0.025 ],
					  [  0.000,  0.050 ],
					  [ -0.050,  0.000 ],
					  [ -0.025,  0.000 ],
					  [  0.025,  0.000 ],
					  [  0.050,  0.000 ] ],
			       dither => [ [ 0.0, 0.0 ],
					   [ 10.5, 0.0 ],
					   [ 10.5, 10.5 ],
					   [ 0.0, 10.5 ] ], 
			       psf_file => "fft11.fits",
			       rrg_dir => "rrg01",
			   }, # End dither_new_psf scheme.

			   dither_old_psf => {
			       pixel_scale => [ 0.04,
						0.05,
						0.06,
						0.07,
						0.08,
						0.09,
						0.10,
						0.11,
						0.12,
						0.13,
						0.14,
						0.15,
						0.16 ],
			       gamma => [ [  0.000, -0.050 ],
					  [  0.000, -0.025 ],
					  [  0.000,  0.000 ],
					  [  0.000,  0.025 ],
					  [  0.000,  0.050 ],
					  [ -0.050,  0.000 ],
					  [ -0.025,  0.000 ],
					  [  0.025,  0.000 ],
					  [  0.050,  0.000 ] ],
			       dither => [ [ 0.0, 0.0 ],
					   [ 10.5, 0.0 ],
					   [ 10.5, 10.5 ],
					   [ 0.0, 10.5 ] ], 
			       psf_file => "SNAP_PSF",
			       rrg_dir => "rrg01",
			   }, # End dither_old_psf scheme.

			   step => {
			       # SNAP baseline.
			       snap => {
				   pixel_scale => [ 0.04,
						    0.01 ],
				   psf_file => [ "step_snap" ],
			       },

			       # SNAP 1.4 m primary.
			       snap_1_4m => {
				   pixel_scale => [ 0.10 ],
				   psf_file => [ "step_snap_1_4m" ],
			       },

			       # Random ACS Tiny Tim PSF.
			       acs_tt => {
				   pixel_scale => [ 0.04,
						    0.10 ],
				   psf_file => [ "step_acs_tt" ],
				   analytic => [ 0,
						 1 ],
			       },

			       # ACS Tiny Tim PSFs with given ellipticities.
			       acs_tt_e => {
				   pixel_scale => [ 0.04 ],
				   psf_file => [ "ei_eq_0",
						 "e1_gt_0",
						 "e2_gt_0",
						 "e1_lt_0",
						 "e2_lt_0" ],
				   n_gals => [ 50,
					       50,
					       50,
					       50,
					       100 ],
			       },
			       
			       # ACS Tiny Tim PSFs that vary across FOV.
			       acs_tt_var => "",
			       
			   }, # End step scheme.
			   ); # End Scheme hash.
    
    foreach my $possibleScheme ( keys %possibleSchemes ) {
	if ($scheme eq $possibleScheme) {
	    $hashRef = $possibleSchemes{$scheme};
	}
    }

    return $hashRef;
} # End sub Scheme.


END {}

1;
