package Simgen::IO;

use strict;
#use warnings;
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::Locale;


BEGIN {
 
    use Exporter;
    our (@ISA, @EXPORT);
 
    @ISA         = qw(Exporter);
    @EXPORT      = qw(message run);
}




sub message {

    foreach ( @_ ) {
	print "simgen: $_\n" if Simgen::Global::verbose;
    }
}

sub run {

    my @cmd = @{$_[0]};
    my $verbose = $_[1];
    message( "Executing @cmd" ) if Simgen::Global::verbose and $verbose;
    my @out = `@cmd 2>&1`;
    foreach ( @out ) {
	print "$_" if Simgen::Global::verbose and $verbose;
    }
}

#
#
# Tweak the various tweaks in almost all the parameter files.
#
#
sub tweak {

    my $t = shift;
    my $a = shift;
    my $paramFiles = shift;

    &printVals($t, $a);

    for my $title ( keys %$paramFiles ) {

	message("Skipping $title initialization.\n") and next if ( $title eq "sh paths" || $title eq "rrg paths" );

	&initweak("$title", $t, $paramFiles);
    }
} # End sub tweak.

#
#
# Initialize only the specified parameter file.
#
#
sub initweak {

    my $title = shift;
    my $t = shift;
    my $paramFiles = shift;

    chomp(my $host = `hostname`);
    my $nRun = "host-${host}_pid-$$";
    my $tmpFile = "$$paramFiles{$title}{file}_${nRun}.tmp";
    run(["cp $$paramFiles{$title}{'default file'} $tmpFile"]);

    for my $tweak ( @{$$paramFiles{$title}{tweaks}} ) {

	unless ( $tweak eq "N_RUN" ) {
	    if ( $$t{$tweak} eq "" ) {
		message("Tweak $tweak hasn't been set *or* $tweak doesn't exist!");
		die;
	    }
	}
	&replaceInFile("${tweak}.SIMGEN","$$t{$tweak}",$tmpFile);
    }
    
    run(["mv $tmpFile $$paramFiles{$title}{file}"]);
    
} # End sub initweak.


#
#
# Verify that at least one simgen variable exists in a file.
#
#
sub verifyFile {
    my $initialString;
    $initialString = $_[0];
    my $paramFile;
    $paramFile = $_[1];

    if (&existsInFile($initialString,$paramFile)) {
#	message("File $paramFile");
#	message("is _probably_ properly initialized with $initialString");
    } else {
	message("File $paramFile not initialized properly.");
	message("Please put $initialString somewhere in it.");
	die;
    }

    unless (&existsInFile($initialString,$paramFile)) {
	die;
    }
} # End sub verifyFile.


#
#
# Replace all instances of $thisString with $nextString in file $inFile.
#
#
sub replaceInFile {

    my $thisString;
    $thisString = $_[0];
    my $nextString;
    $nextString = $_[1];
    my $inFile;
    $inFile = $_[2];

    &verifyFile($thisString,$inFile);

    chomp(my $host = `hostname`);
    my $nRun = "host-${host}_pid-$$";
    my $tmpFile = "${nRun}.tmp";
    my $inline;

# Open the desired files for input and output.
    unless (open(IN,$inFile)) {
	message("Cannot open file $inFile for reading.");
	die;
    }
    unless (open(OUT,">$tmpFile")) {
	message("Cannot open temporary file $tmpFile for writing.");
	die;
    }

    message("Setting $thisString to $nextString...");
    message("...in $inFile");
# Replace (squeeze) each instance of $initialString with the first section number in each line,
# then ouput all lines to OUT file.
    while ($inline = <IN>) {
	$inline =~ s/$thisString/$nextString/g;
	print OUT $inline;
    }
# Close all open files.
    close(IN);
    close(OUT);
# Swap dummy $tmpFile to original $inFile.
#    unless (rename($tmpFile,$inFile)) {
#	message("Can't rename $tmpFile to $inFile.");
#	die;
#    }
    `mv $tmpFile $inFile`;
} # End sub replaceInFile.


#
#
# Check for at least one instance of $stringToCheck in $file.
#
#
sub existsInFile {

    my $stringToCheck;
    $stringToCheck = $_[0];
    my $file;
    $file = $_[1];

    my $goodFile;

    unless (open(IN,$file)) {
	message("Can't open $file for reading");
	die;
    }
    $goodFile = false;
    while (<IN>) {
	if (/$stringToCheck/) {
	    $goodFile = true;
	}
    }
    close(IN);

# Return this value.
    $goodFile;
} # End sub existsInFile.


#
#
# Message the values of the main hashes.
#
#
sub printVals {

    my $t = shift;
    my $a = shift;

    message("Tweaks...");
    for my $key ( sort keys %$t ) {
	message( "\t$key \t= $$t{$key}" );
    }

    message("Commandline arguments...");
    for my $key ( sort keys %$a ) {
	if ( $key eq "gamma" ) {
	    message( "\t$key \t= [ $$a{$key}[0], $$a{$key}[1] ]" );
	} else {
	    message( "\t$key \t= $$a{$key}" );
	}
    }
#    for my $key1 ( sort keys %paramFiles ) {
#	for my $key2 ( keys %{$paramFiles{$key1}} ) {
#	    message( "$key1 $key2 = $paramFiles{$key1}{$key2}" );
#	}
#    }
} # End sub printVals.



sub openLog {

    my $logFile = $_[0];
    
#
# Prepare log file.
#
    if (-f $logFile) {
	my @existingLogs = `ls -1 ${logFile}*`;
	my $suffix = $#existingLogs + 1;
	rename $logFile, $logFile . ".$suffix.old";
    }
    open(LOG,">>$logFile") || die "simgen: Can't open $logFile for logging output.\n";

    $logFile;
} # End sub openLog.


#
#
# Initialize the log file.
#
#
sub initLog {

    my $paramFiles = shift;
    my %paramFiles = %$paramFiles;

    chomp(my $when = `date`);
    print LOG "\n\n*** NEW SESSION: $when ***\n\n";
    print LOG "simgen: Following are the runtime parameters used to get the subsequent output.\n";
    print LOG "simgen: They come from the script you fed into the executable.\n";
    
    for my $key ( keys %paramFiles ) {

	if ( $paramFiles{$key}{"default file"} ) {

	    print LOG "simgen: Begin default parameter file $paramFiles{$key}{'default file'}.\n\n";
	    unless ( open( IN,$paramFiles{$key}{"default file"} ) ) {
		&message("Can't open $paramFiles{$key}{'default file'} for reading");
		die;
	    }

	    while (<IN>) {
		print LOG $_;
	    }

	    print LOG "\nsimgen: End default parameter file $paramFiles{$key}{'default file'}.\n";
	    close(IN);
	}
    }
} # End sub initLog.


sub MakeRTScript {

    my $algo = shift;
    my $algoMethod = shift;
    my $software = shift;
    my %software = %$software;
    my $paramFiles = shift;
    my %paramFiles = %$paramFiles;
    my $t = shift;
    my %t = %$t;
    my $prefix = shift;

    my $param = "$algo param";
    die "Unable to get identifier for set of parameters needed by $algo $algoMethod\n" unless $param;
    my $scriptFilename = $software{$algo}{$algoMethod}{'exec name'};
    die "Unable to locate name of script template for $algo $algoMethod\n" unless $scriptFilename;
    die "Need to specify a prefix" if ( $prefix eq "" );
    $scriptFilename = "$t{'data_out_home'}/${prefix}_$scriptFilename";
    my $logFilename = "${scriptFilename}.o.log";
    system "mv -f $scriptFilename $t{'data_out_home'}/old" if ( -e $scriptFilename );
#    system "mv -f $logFilename $t{'data_out_home'}/old" if ( -e $logFilename );

#    message("Making runtime script $scriptFilename for $algo $algoMethod");
    message("Making $prefix for $algo $algoMethod");
    open( SCRIPT , "> $scriptFilename" ) || die "Can't open script file ", $scriptFilename, " for writing";

    print SCRIPT "#!/bin/tcsh\n";
#    print SCRIPT "#\$ -l h_cpu=10:00:00 -cwd -o ${scriptFilename}.o.log -e ${scriptFilename}.e.log\n\n";
    print SCRIPT "#\n";
    print SCRIPT "# NERSC's PDSF SGE qsub arguments:\n";
#    print SCRIPT "#\$ -l h_cpu=10:00:00\n";
#    print SCRIPT "#\$ -l dv22io=50\n";
    print SCRIPT "#\$ -cwd\n";
    print SCRIPT "#\$ -v SIMGEN\n";
    print SCRIPT "#\$ -o $logFilename -j y\n";
    print SCRIPT "\n";
    print SCRIPT "echo hostname: `hostname`\n";
    print SCRIPT "echo script: $scriptFilename\n";
    print SCRIPT "echo starttime: `date`\n";
    print SCRIPT "# Start a stopwatch.\n";
    print SCRIPT "set simgen_tick = `tick`\n";
    print SCRIPT "# Load needed modules (PDSF)\n";
    if (Simgen::Global::Institute eq "PDSF") {
	print SCRIPT "module load IDL\n";
    }
    print SCRIPT "# Set environment variables.\n";
    if (Simgen::Global::Institute eq "PDSF") {
	print SCRIPT ("setenv LM_LICENSE_FILE /usr/local/pkg/IDL-6.0/license/license.dat\n");
    }
    foreach my $tweak ( @{$paramFiles{$param}{tweaks}} ) {
	#die "Tweak $tweak doesn't exist or hasn't been initialized\n" if ( $t{$tweak} eq "" );
	print SCRIPT ("setenv simgen_", $tweak, " \t ", $t{$tweak}, "\n");
    }
    my $paths = "$algo paths";
    foreach my $pathsTweak ( @{$paramFiles{$paths}{tweaks}} ) {
	die "Tweak $pathsTweak doesn't exist or hasn't been initialized\n" if ( $t{$pathsTweak} eq "" );
	print SCRIPT ("setenv simgen_", $pathsTweak, " \t ", $t{$pathsTweak}, "\n");
    }
    my $dot_sav = $software{$algo}{$algoMethod}{exec};
    print SCRIPT ("\nif ( ! -d \$simgen_data_out_home ) mkdir -p \$simgen_data_out_home\n");
    print SCRIPT ("\nif ( ! -d \$simgen_tmpdir ) mkdir -p \$simgen_tmpdir\n");
#    print SCRIPT ("$t{'home'}/wait4license.pl\n");
#    print SCRIPT ("\nnice +16 idl -queue -rt=${dot_sav}.sav\n\n");
    print SCRIPT ("\nidl -queue -rt=${dot_sav}.sav\n\n");
    if ( $algoMethod eq "simage" ) {
#	print SCRIPT ("mv \$simgen_data_in_home/$prefix* \$simgen_data_out_home\n\n");
    }
    print SCRIPT ("unsetenv LM_LICENSE_FILE\n") if (Simgen::Global::Institute eq "PDSF");
    foreach my $tweak ( @{$paramFiles{$param}{tweaks}} ) {
	print SCRIPT ("unsetenv simgen_", $tweak, "\n");
    }
    foreach my $pathsTweak ( @{$paramFiles{$paths}{tweaks}} ) {
	print SCRIPT ("unsetenv simgen_", $pathsTweak, "\n");
    }
    print SCRIPT "tock \$simgen_tick\n";
    print SCRIPT "unset simgen_tick\n";
    print SCRIPT "echo stoptime: `date`\n";

    close SCRIPT;

    run(["chmod +x $scriptFilename"], 0);

}

sub SetDefaultTweaks {

    my $filename = shift;

    my %tweaks;

    open(PARAM, "< $filename") || die("Can't open $filename: $!");
    my $var;
    my $nested = 0;
    my $firstNested = 1;
    LINE: foreach my $line ( <PARAM> ) {
	chomp($line);
	# Remove leading and trailing whitespace.
	$line =~ s/^\s*((\s*\S+)*)\s*$/$1/;
	# Uncomment
	$line =~ s/\043.*$//; # \043 == '#'
	# Continue if null.
	next LINE if $line =~ /^$/;
	if ( !$nested ) {
	    # Get the first word.
	    /^(\w+)(.*)/;
	    $var = $1;
	    my $theRest = $2;
	    # Remove leading and trailing whitespace.
	    $theRest =~ s/^\s*//;
	    $theRest =~ s/\s*$//;
	    # Check for a '{' as first character.
	    $nested = 1, $firstNested = 1 if $theRest =~ /^\{/;
	    unless ( $nested ) {
		# Get the non-nested values.
		if ( $theRest =~ /^$/ ) {
		    $tweaks{$var} = undef;
		} else {
		    $tweaks{$var} = $theRest;
		}
	    }
	} else {
	    my $theRest = $line;
	    # Check for a '{'.
	    die "Need to close brackets before openning a new one" if $theRest =~ /\{/;
	    # Check for a '}' as first character.
	    $nested = 0 if $theRest =~ /^\}/;
	    if ( $nested ) {
		# Get the nested values.
		if ( $theRest =~ /^$/ ) { # If null string.
		    $tweaks{$var} = undef;
		} else { # Else if not null string.
		    if ( $firstNested ) {
			$tweaks{$var} = [ ($theRest) ];
			$firstNested = 0;
		    } else {
			push @{ $tweaks{$var} }, $theRest;
		    } # End if firstNested.
		} # End if null string.
	    } # End if nested.
	} # End if not nested.
    }
    die "Need to close brackets" if $nested;
    close(PARAM);

    # Set a couple by hand.
    $tweaks{home} = Simgen::Locale::Home('lib');
    $tweaks{data_home} = Simgen::Locale::Home('data');

    return \%tweaks;
}


sub GetTweaks {

    my $filename = shift;
    my $t = shift;
    my %tweaks = %$t;
    $filename =~ /((.*\/)|)(\S+)\.param/;
    if ( ! -f $filename ) {
	message "Scheme '$3' doesn't exist,";
	my $dir = $1;
	$dir = '.' if $dir =~ /^$/;
	$dir =~ s/(^.+)\//$1/;
	message "checked in directory '$dir'.";
	die;
    }
    open(PARAM, "< $filename") || die("Can't open $filename: $!");
    my $var;
    my $nested = 0;
    my $firstNested = 1;
    LINE: foreach my $line ( <PARAM> ) {
	chomp($line);
	# Uncomment
	$line =~ s/\043.*$//; # \043 == '#'
	# Remove leading and trailing whitespace.
	$line =~ s/^\s*((\s*\S+)*)\s*$/$1/;
	# Continue if null.
	next LINE if $line =~ /^\s*$/;
	if ( !$nested ) {
	    # Get the first word.
	    $line =~ /^(\w+)\s*(.*)/;
	    $var = $1;
	    my $theRest = $2;
	    # Remove leading and trailing whitespace.
	    $theRest =~ s/^\s*//;
	    $theRest =~ s/\s*$//;
	    # Check for a '{' as first character.
	    $nested = 1, $firstNested = 1 if $theRest =~ /^\{/;
	    unless ( $nested ) {
		# Get the non-nested values.
		unless ( $theRest =~ /^$/ ) {
		    $tweaks{$var} = $theRest;
		}
	    }
	} else {
	    my $theRest = $line;
	    # Check for a '{'.
	    die "Need to close brackets before openning a new one" if $theRest =~ /\{/;
	    # Check for a '}' as first character.
	    $nested = 0 if $theRest =~ /^\}/;
	    if ( $nested ) {
		# Get the nested values.
		unless ( $theRest =~ /^$/ ) { # Null string.
		    if ( $firstNested ) {
			$tweaks{$var} = [ $theRest ];
			$firstNested = 0;
		    } else {
			push @{ $tweaks{$var} }, $theRest;
		    } # End if firstNested.
		} # End if null string.
	    } # End if nested.
	} # End if not nested.
    }
    die "Need to close brackets" if $nested;
    close(PARAM);

    return \%tweaks;
}


END {}

1;
