package Simgen::Init;

use strict;
use warnings;
use Simgen::SuperGlobal;
use Simgen::Locale;
use Simgen::IO;

sub N_Pixels {

    my $pixel_scale = shift;
    my @pixel_scale = @$pixel_scale;
    my $imageSize = shift;
    
    my @n_pixels;

    if ( $imageSize =~ /\barcsec$/ ) {
	# Strip the arcsec units.
	$imageSize =~ s/\s+arcsec//;
	for my $i (0 .. $#pixel_scale) {
	    $n_pixels[$i] = int( $imageSize /$pixel_scale[$i] );
	}
    } else {
	for my $i (0 .. $#pixel_scale) {
	    $n_pixels[$i] = int( 4096 * 0.04 / $pixel_scale[$i] );
	}
    }
    
    return @n_pixels;
}

sub Min_Rad {

    my $pixel_scale = shift;
    my @pixel_scale = @$pixel_scale;
    my $minRadSize = shift;

    my @minRadSize = @{$minRadSize};

    my @min_rad;

    if ( $#minRadSize == 0 ) {
	$minRadSize = $minRadSize[0];
	if ( $minRadSize =~ /\barcsec$/ ) {
	    # Strip the arcsec units.
	    $minRadSize =~ s/\s+arcsec//;
	    for my $i (0 .. $#pixel_scale) {
		$min_rad[$i] = $minRadSize / $pixel_scale[$i];
	    }
	} else {
	    for my $i (0 .. $#pixel_scale) {
		$min_rad[$i] = $minRadSize;
	    }
	}
    } else {
	die "Specify one min_rad per pixel scale" if $#minRadSize != $#pixel_scale;
	for my $i (0 .. $#pixel_scale) {
	    if ( $minRadSize[$i] =~ /\barcsec$/ ) {
		# Strip the arcsec units.
		$minRadSize[$i] =~ s/\s+arcsec//;
		for my $i (0 .. $#pixel_scale) {
		    $min_rad[$i] = $minRadSize[$i] / $pixel_scale[$i];
		}
	    } else {
		for my $i (0 .. $#pixel_scale) {
		    $min_rad[$i] = $minRadSize[$i];
		}
	    }
	}
    }
	
    return @min_rad;
    
}

sub Tweaks {

    my %t = (
# Simage parameters.
	     telescope         => "SNAP",
	     exposure_time     => 10.0,
	     n_pixels          => "",
	     pixel_scale       => "",
	     n_gal             => "",
	     n_star            => "",
	     stellar           => "",
	     x_dither          => "",
	     y_dither          => "",
	     sigma_epsilon     => 0.3,
	     random_seed       => "",
	     noise_seed        => "",
	     random_positions  => true,
	     background        => 0.0,
	     shot_noise        => 0.002,
	     gauss_noise       => "0.000035",
	     gain              => 14.0,
	     sat_level         => 4097.0,
	     read_noise        => 5.0,
	     DQE               => 0.40,
	     bias              => 300.0,
	     hdf_time          => 100.0,
	     CCDwidth          => 10.5,
	     CCDthickness      => 200.0,
	     CRdiffusion       => false,
	     CRflux            => 4.5,
	     CRn_elec          => 70.0,
	     CRsigma           => 3.5,
	     smooth            => false,
	     kernel            => 2.0,
	     positive_definite => true,
	     diff_length       => 4.0,
	     "run"             => "",
	     output_file       => "dummy",
	     input_gamma_1     => "",
	     input_gamma_2     => "",
	     input_catalogue   => "F",
	     const_psf_size    => false,
	     circularize_psf   => false,

# RRG parameters.
	     star_run          => "",
	     mult              => 2,
	     min_rad           => 3.5,
	     sn                => 10,
	     d_cut_lower       => -1,
	     d_cut_upper       => 2000,
	     sex_kernel        => "default", # none, default, tailored

# *paths.pro parameters.
	     "home"            => Simgen::Locale::Home('lib'),
	     data_in_home      => "",
	     psf_file          => "",
	     thispsf           => "",
	     sex_path          => Simgen::Locale::SexPath,

# Drizzle parameters.
	     dither_fin0       => "",
	     dither_xsh0       => "",
	     dither_ysh0       => "",
	     dither_fin1       => "",
	     dither_xsh1       => "",
	     dither_ysh1       => "",
	     dither_fin2       => "",
	     dither_xsh2       => "",
	     dither_ysh2       => "",
	     dither_fin3       => "",
	     dither_xsh3       => "",
	     dither_ysh3       => "",
	     dither_fout       => "",
	     dither_scale      => "",
	     dither_pfract     => 0,
	     dither_kernel     => "point",
	     drz_dir           => "",

# Simgen parameters.
	     data_home         => Simgen::Locale::Home('data'),
	     data_out_home     => "",
	     );

# All unset tweaks are more slippery and should be set in the main
# compiling script.

    return %t;

} # End sub Tweaks.


sub SetDefaultTweaks {

    my %t = Simgen::Init::Tweaks();
    return \%t;
}





#
#
# Initialize the software settings.
#
#

sub Software {
    
    my $t = shift;
    my %t = %$t;
    
#
#
# Here are all implemented software packages.
#
#
    my %software = (
		    shapelets => {
			simage => {
			    exec => "${t{'home'}}/simage/generate_simage",
			    "exec name" => "generate_simage",
			    home => "${t{'home'}}/simage",
#			    params => "sh param",
			},
			"make simage rt" => {
			    exec => "${t{'home'}}/simage/generate_simage",
			},
		    },
		    rrg => {
			psf => {
			    exec => "${t{'home'}}/rrg/rrg_get_psf",
			    "exec name" => "rrg_get_psf",
			    home => "${t{'home'}}/rrg",
#			    params => "rrg param",
			},
			shear => {
			    exec => "${t{'home'}}/rrg/rrg_get_shear",
			    "exec name" => "rrg_get_shear",
			    home => "${t{'home'}}/rrg",
#			    params => "rrg param",
			},
		    },
		    utils => {
			make_rt => {
			    exec => "simgen_make_rt",
			    shapelets => {
				simage => "${t{'home'}}/simage/generate_simage.pro"
				},
				    rrg => {
					psf =>"${t{'home'}}/rrg/rrg_get_psf.pro",
					shear => "${t{'home'}}/rrg/rrg_get_shear.pro"
					},
					    drizzle => {
						hook =>"${t{'home'}}/IDL_Drizzle_Blot/drizzle_rt.pro",
					    }
			},
		    },
		    drizzle => {
			hook => {
			    home => "${t{'home'}}/IDL_Drizzle_Blot",
			    exec => "${t{'home'}}/IDL_Drizzle_Blot/drizzle_rt",
			    "exec name" => "drizzle_rt",
			}
		    }
		    ); # End software hash.
}


#
#
# Here are all parameter files used by any software package.
#
#
sub ParamFiles {

    my $t = shift;
    my %t = %$t;

    my %paramFiles = (
		      "shapelets param" => {
			  "template" => "${t{'home'}}/simage/simage_parameters.simgen",
			  file	  => "${t{'home'}}/simage/simage_parameters",
			  tweaks => [ "name",
				      "description",
				      "telescope",
				      "exposure_time",
				      "noise_only",
				      "n_pixels",
				      "pixel_scale",
				      "object_type",
				      "n_gal",            
				      "n_star",           
				      "stellar",          
				      "dither_scale",
				      "x_dither",
				      "y_dither",
				      "sigma_epsilon",    
				      "random_seed",      
				      "noise_seed",       
				      "random_positions", 
				      "throughput",
				      "hide_shear",
				      "background",       
				      "CCDwidth",         
				      "CCDthickness",     
				      "CRdiffusion",      
				      "CRflux",           
				      "CRn_elec",         
				      "CRsigma",          
				      "positive_definite",
				      "diff_length",      
				      "run",              
				      "output_file",      
				      "input_gamma_1",      
				      "input_gamma_2",      
				      "input_catalogue",
				      #"psf_file",
				      "psf_name",
				      "const_psf_size",
				      "circularize_psf",
				      "data_home",
				      "data_out_home",
				      "data_in_home",
				      "tmpdir" ]
				      },
		      "shapelets paths" => {
			  "template" => "${t{'home'}}/shapelets/shapelets_paths.pro.simgen",
			  "file" => "${t{'home'}}/shapelets/shapelets_paths.pro",
			  "tweaks" => [ "home",
					"data_in_home",
					"psf_file",
					"thispsf",
					"sex_path" ]			  
					},
		      "shapelets conv" => {
			  file	  => "${t{'home'}}/shapelets/shextractor/config/${t{'telescope'}}.conv",
		      },
		      "rrg param" => {
			  "template" => "${t{'home'}}/rrg/rrg_parameters.simgen",
			  file	  => "${t{'home'}}/rrg/rrg_parameters",
			  tweaks => [ "telescope",
				      "exposure_time",
				      "n_pixels",
				      "pixel_scale",      
				      "n_gal",            
				      "n_star",           
				      "stellar",          
				      "x_dither",
				      "y_dither",
				      "dither_scale",
				      "sigma_epsilon",    
				      "random_seed",      
				      "noise_seed",       
				      "random_positions", 
				      "background",       
				      "CCDwidth",         
				      "CCDthickness",     
				      "CRdiffusion",      
				      "CRflux",           
				      "CRn_elec",         
				      "CRsigma",          
				      "positive_definite",
				      "diff_length",      
				      "run",              
				      "output_file",      
				      "input_gamma_1",      
				      "input_gamma_2",      
				      "input_catalogue",
				      "psf_file",
				      "thispsf",
				      "psf_name",
				      "const_psf_size",
				      "circularise_psf",
				      "data_home",
				      "star_run",
				      "mult",
				      "min_rad",
				      "sn",
				      "d_cut_lower",
				      "d_cut_upper",
				      "data_out_home",
				      "sex_kernel",
				      "tmpdir" ],
		      },
		      "rrg paths" => {
			  "template" => "${t{'home'}}/rrg/rrg_paths.pro.simgen",
			  "file" => "${t{'home'}}/rrg/rrg_paths.pro",
			  "tweaks" => [ "home",
					"data_in_home",
					"psf_file",
					"thispsf",
					"sex_path" ]			  
					},
		      "rrg conv" => {
			  file	  => "${t{'home'}}/shapelets/shextractor/config/${t{'telescope'}}.sex",
		      },
		      "drizzle param" => {
			  file => "${t{'home'}}/IDL_Drizzle_Blot/drizzle_parameters",
			  tweaks => [ "dither_fin0",
				      "dither_xsh0",
				      "dither_ysh0",
				      "dither_fin1",
				      "dither_xsh1",
				      "dither_ysh1",
				      "dither_fin2",
				      "dither_xsh2",
				      "dither_ysh2",
				      "dither_fin3",
				      "dither_xsh3",
				      "dither_ysh3",
				      "dither_fout",
				      "dither_scale",
				      "dither_pfract",
				      "dither_kernel",
				      "drz_dir",
				      "data_in_home",
				      "data_out_home", 
				      "tmpdir",
				      "sex_path", ]
				      },
		      ); # End paramFiles hash.
}


sub RandomizeGamma {

    die "You can't use this function anymore";

    my %t = %{ $_[0] };

    my $total = $t{gamma}[0];
    # Remove "random" keyword.
    $total =~ s/\s*random\s*//;
    # Make sure it's an integer.
    $total = int $total;

    ( my $lower, my $upper ) = split( /\s+/, $t{gamma_random_range} );
    my $mean = abs( ( $upper + $lower ) / 2.0 );

    srand( $t{gamma_seed} );

    my @gamma = undef;
    my @signCount = ( 0, 0 );
    foreach my $i ( 0 .. $total-1 ) {

	my $gamma_1;
	{ # Gamma_1.

	    # Randomly choose whether to left or right of mean.  Allows
	    # for negative numbers, which the random number generator
	    # cannot generate.
	    
	    my $rand = rand(1);
	    my $sign;
	    $sign = "-", $signCount[0]++ if $rand < 0.5;
	    $sign = "+", $signCount[1]++ if $rand >= 0.5;
	    
	    $gamma_1 = $mean + rand( abs( $upper ) ) if $sign eq "+";
	    $gamma_1 = $mean - rand( abs( $upper ) ) if $sign eq "-";

	}


	my $gamma_2;
	{ # Gamma_2.

	    # Randomly choose whether to left or right of mean.  Allows
	    # for negative numbers, which the random number generator
	    # cannot generate.
	    
	    my $rand = rand(1);
	    my $sign;
	    $sign = "-", $signCount[0]++ if $rand < 0.5;
	    $sign = "+", $signCount[1]++ if $rand >= 0.5;
	    
	    $gamma_2 = $mean + rand( abs( $upper ) ) if $sign eq "+";
	    $gamma_2 = $mean - rand( abs( $upper ) ) if $sign eq "-";

	}

	if ( $i == 0 ) {
	    @gamma = "$gamma_1 $gamma_2";
	} else {
	    push @gamma, "$gamma_1 $gamma_2";
	} # End if first.


    }

    return @gamma;
}

sub DummyGamma {

    my %t = %{ $_[0] };

    my $totalGammas = $t{gamma}[0];
    $totalGammas =~ s/\s*random\s*//;
    $totalGammas = int $totalGammas;

    my @gamma = undef;

    for ( my $i = 0; $i < $totalGammas; $i++ ) {
	$gamma[$i] = $i;
    }

    return @gamma;
}

sub GetARandomGamma {

    my %t = %{ $_[0] };
    
    ( my $lower, my $upper ) = split( /\s+/, $t{gamma_random_range} );
    my $mean = abs( ( $upper + $lower ) / 2.0 );
    
    srand( $t{gamma_seed} );
    
    my @gamma = undef;
    
    { # Gamma_1.
	
	# Randomly choose whether to left or right of mean.  Allows
	# for negative numbers, which the random number generator
	# cannot generate.
	
	my $rand = rand(1);
	my $sign;
	$sign = "-" if $rand < 0.5;
	$sign = "+" if $rand >= 0.5;
	
	$gamma[0] = $mean + rand( abs( $upper ) ) if $sign eq "+";
	$gamma[0] = $mean - rand( abs( $upper ) ) if $sign eq "-";
	
    }
    
    
    { # Gamma_2.
	
	# Randomly choose whether to left or right of mean.  Allows
	# for negative numbers, which the random number generator
	# cannot generate.
	
	my $rand = rand(1);
	my $sign;
	$sign = "-" if $rand < 0.5;
	$sign = "+" if $rand >= 0.5;
	
	$gamma[1] = $mean + rand( abs( $upper ) ) if $sign eq "+";
	$gamma[1] = $mean - rand( abs( $upper ) ) if $sign eq "-";
	
    }
    
    return "@gamma";

}

END {};

1;
