package Simgen::Locale;

use strict;
use warnings;
use Simgen::SuperGlobal;
use Simgen::Global;

sub Licenses {

    my $keyword = shift;

    my %h = ( 
	      total     => '',
	      available => '',
	      );

    my %KnownLicenseSchemes = (
			       Caltech => {
				   total     => 250,
				   available => 70,
			       },
			       PDSF => {
				   total     => 768,
				   available => 768,
#				   total     => 120,
#				   available => 60,
			       },
			       );
			  
    for my $institute ( keys %KnownLicenseSchemes ) {
	$h{$keyword} = $KnownLicenseSchemes{$institute}{$keyword} if $institute eq Simgen::Global::Institute;
    }

    die "Unknown license keyword $keyword" unless $h{$keyword};

    return $h{$keyword};
}

sub Home {

    my $keyword = shift;

    my %val = (
	       data => '',
	       lib  => ''
	       );
    
    my %KnownHomes = (
		      Caltech => {
			  data => "/scr/fwh/data",
			  lib  => "/home/elen/fwh/simgen",
		      },
		      PDSF    => {
			  data => "/eliza6/astrogfs/davej/simgen_data",
			  lib  => "/common/amanda/davej/code",
		      },
		      );

    for my $institute ( keys %KnownHomes ) {
	$val{$keyword} = $KnownHomes{$institute}{$keyword} if $institute eq Simgen::Global::Institute;
    }

    die "Unknown home keyword $keyword" unless $val{$keyword};

    return $val{$keyword};
}

sub IDLdir {

    my $val = "";

    my %KnownIDLdirs = (
			Caltech => "/usr/local/rsi/idl_6.0",
			PDSF => "/usr/local/pkg/IDL-6.0/idl_6.0"
			);

    for my $institute ( keys %KnownIDLdirs ) {
	$val = $KnownIDLdirs{$institute} if $institute eq Simgen::Global::Institute;
    }

    die("IDL directory location unknown at ", Simgen::Global::Institute) unless $val;

    return $val;
}

sub SexPath {

    return &SexDir();
}
sub SexDir {

    my $val = "";

    my %KnownSexDirs = (
			Caltech => "/usr/local/optical/sextractor/bin/sex",
			PDSF => "/common/amanda/fits/sextractor-2.3.2/src/sex",
			);

    for my $institute ( keys %KnownSexDirs ) {
	$val = $KnownSexDirs{$institute} if $institute eq Simgen::Global::Institute;
    }

    die("Location of sex executable unknown at ", Simgen::Global::Institute) unless $val;

    return $val;
}

sub Cluster {

    if ( Simgen::Global::Institute eq "PDSF" ) {
	return "SGE";
    } elsif ( Simgen::Global::Institute eq "Caltech" ) {
	return "";
    }

    return "";
}

END { }

1;
