package Simgen::Util;

use strict;
use warnings;
use Simgen::IO;


#
#
# Subroutines tick and tock together are kind of a stopwatch.
# &tock(@tick) tells you the elapsed time since @tick =
# &tick("unique stopwatch id").
#
#
sub tick {
    
    my $id = shift;
    my $silent = shift;

    chomp(my $date = `date`);
    message("Starting stopwatch '$id' on ${date}.") unless $silent;

    chomp( my $tick = `date +%s` );

    my @tick = ($tick, $id);
} # End sub tick.

sub tock {

    my $tick = shift;
    my $id = shift;
    my $silent = shift;

    chomp(my $date = `date`);
    message("Stopping stopwatch '$id' on ${date}.") unless $silent;

    chomp( my $tock = `date +%s` );

    my $runtime = $tock - $tick;
    (my $hours, my $minutes, my $seconds) = &sec2hms($runtime);
    message("Elapsed time = $runtime sec, or $hours:$minutes:$seconds hr:min:sec");
} # End sub tock.


#
#
# Convert seconds to hours:minutes:seconds.
#
#
sub sec2hms {
    my $runtime = $_[0];

    my $hours = $runtime / 3600;
    $hours = int($hours);
    my $remainder = $runtime % 3600;
    my $minutes = $remainder / 60;
    $minutes = int($minutes);
    my $seconds = ($remainder % 60) + (($runtime * 100) % 100) / 100;

    ($hours, $minutes, $seconds);
} # End sub sec2hms.

sub nDigits {

    my $n = shift;
    my $val = 1;
    my $shift = int($n/10);
    if ( $shift > 0 ) {
        $val = nDigits( $shift ) + 1;
    }
    return $val;
}

sub Suffix {

    my $count = shift;

    my $nZeros = 4 - nDigits($count);
    my $zeros = "";
    foreach ( 1 .. $nZeros ) { $zeros = "${zeros}0"; }
    return "r$zeros$count";
    
}

#
#
# Prefix zeros to the number num so that it has maxDigits digits.
#
#
sub prefixZeros {

    my $num = shift;
    my $maxDigits = shift;

    my $nZeros = $maxDigits - nDigits($num);
    my $zeros = "";
    foreach ( 1 .. $nZeros ) { $zeros = "${zeros}0"; }
    return "$zeros$num";
} # End subroutine prefixZeros.
 
sub rootFilename {

    my $num = shift;
    my $maxDigits = shift;
    my $algo = shift;
    my $prefixOnly = shift;
    my $prefix = shift;

    if ( $algo eq "rrg" || $algo eq "shapelets" ) {
	$prefix = "simrun" unless $prefix;
    } elsif ( $algo eq "drizzle" ) {
	$prefix = "simdrz" unless $prefix;
    } else {
	die "Invalid algorithm $algo";
    }

    return $prefix if $prefixOnly;
    return $prefix . prefixZeros($num,$maxDigits);
}

sub drzFilename {

    my $num = shift;
    my $maxDigits = shift;

    return "simdrz" . prefixZeros($num,$maxDigits);
}

END {}

1;

