#!/usr/bin/perl

use strict;
use warnings;
die "Please set the SIMGEN environment variable" unless $ENV{SIMGEN};
use lib "$ENV{SIMGEN}";
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::IO;
use Simgen::Init;
use Simgen::Util;

if ($#ARGV != 6) {
    print "Usage: simfile.pl psf pixel_scale catalogue \\\n\tgamma_1 gamma_2 stellar? exposure_time\n";
    die;
}

my $psf_arg = $ARGV[0];
my $pixel_scale_arg = $ARGV[1];
my $input_catalogue_arg = $ARGV[2];
my @input_gamma_arg = @ARGV[3,4];
my $stellar_arg = $ARGV[5];
my $exposure_time_arg = $ARGV[6];

# Initialize variables from global libraries.
my %t = Simgen::Init::Tweaks;
my %paramFiles = Simgen::Init::ParamFiles( \%t );
my %software = Simgen::Init::Software( \%t );
my @pixel_scale = @Simgen::Global::pixel_scale;
my @n_pixels = Simgen::Init::N_Pixels( \@pixel_scale );
my @gamma = @Simgen::Global::gamma;
my @dither = @Simgen::Global::dither;


use FileHandle;
use IPC::Open2;

my $pid = open2(*Reader, *Writer, "idl" );
print Writer "out='$psf_arg'\n";
print Writer "simage_make_filename,out,pixel_scale=$pixel_scale_arg,input_catalogue='$input_catalogue_arg',input_gamma=[$input_gamma_arg[0],$input_gamma_arg[1]],stellar=$stellar_arg,exposure_time=$exposure_time_arg\n";
print Writer "print,out\n";
my $file = <Reader>;
chomp($file);
print $file,"\n";
