#!/usr/local/bin/perl





use strict;
use warnings;
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::Util;
use Simgen::IO;
use Simgen::Init;
use Simgen::Engine;





my @globalTick = Simgen::Util::tick("simages global stopwatch");

my %t = Simgen::Init::Tweaks( \%Simgen::Global::t );
my %a = Simgen::Init::Args( \%Simgen::Global::a, \%Simgen::Software::paramFiles );
my %paramFiles = Simgen::Init::ParamFiles( \%t );
my %software = Simgen::Init::Software( \%t );

my @pixel_scale = @Simgen::Global::pixel_scale;
my @n_pixels = @Simgen::Global::n_pixels;
my @gamma = @Simgen::Global::gamma;
my @dither = @Simgen::Global::dither;



chdir("$t{DATA_IN_HOME}") || die "chdir $t{DATA_IN_HOME}: $!";
run("pwd");

if ( Simgen::Global::want2do eq "simages" ) {

    message "Performing simages";

    my $simagesCount = 0;
    for ( $t{RANDOM_SEED} = 1; $t{RANDOM_SEED} < 2; $t{RANDOM_SEED}++ ) {

	my $pxIndex = 0;
	for my $pixel_scale ( @pixel_scale ) {
	    $t{PIXEL_SCALE} = $pixel_scale;
	    
	    if ( $pixel_scale == 0.04 ) {
		Simgen::IO::initweak("sh paths", \%t, \%paramFiles);
		  Simgen::IO::initweak("rrg paths", \%t, \%paramFiles);
	      }

	    my $needStarField = true;

	    run("rm *_${t{N_RUN}}*");

	    for my $gamma ( @gamma ) {

		for my $dither ( @dither ) {
		    
		    $simagesCount++;
		    $t{NOISE_SEED} = $simagesCount;

		    if ($needStarField) { # Begin star field routine.

			message("Simulating a globular cluster using simages.\n");

			my @tick = Simgen::Util::tick("local stopwatch");

			{ # ... and forth.
			    $t{GAMMA_1} = 0;
			    $t{GAMMA_2} = 0;
			    $a{"sh param file"} = $paramFiles{"sh param"}{file};
			    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
			    $t{D_CUT_LOWER} = 1.2 * sqrt($factor * $factor + 0.36);
			    
			    $t{N_PIXELS} = $n_pixels[$pxIndex];
			    $t{X_DITHER} = @$dither[0];
			    $t{Y_DITHER} = @$dither[1];
			    $t{N_GAL}	 = 0;
			    $t{N_STAR}	= 372;
			    $t{STELLAR} = true;
			    $t{CONST_PSF_SIZE} = 1;
			}

			Simgen::IO::tweak(\%t, \%a, \%paramFiles);
			Simgen::Engine::Run("shapelets","simages", \%software, \%a);

			{ # Back...

			    $t{GAMMA_1} = "";
			    $t{GAMMA_2} = "";
			    $a{"sh param file"} = "";
			    $t{D_CUT_LOWER} = "";
			    
			    $t{N_PIXELS} = "";
			    $t{X_DITHER} = "";
			    $t{Y_DITHER} = "";
			    $t{N_GAL}	= "";
			    $t{N_STAR}	= "";
			    $t{STELLAR} = "";
			    $t{CONST_PSF_SIZE} = "";
			}

			Simgen::Util::tock(@tick);
			
			$needStarField = false;
		    } # End star field routine.

		    if (true) { # Begin galaxy field routine.

			message("Simulating a galaxy field using simages.\n");
			
			my @tick = Simgen::Util::tick("local stopwatch");
			
			{ # ...and forth.
			    $t{GAMMA_1} = @$gamma[0];
			    $t{GAMMA_2} = @$gamma[1];
			    $a{"sh param file"} = $paramFiles{"sh param"}{file};
			    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
			    $t{D_CUT_LOWER} = 1.2 * sqrt($factor * $factor + 0.36);
			    
			    $t{N_PIXELS} = $n_pixels[$pxIndex];
			    $t{X_DITHER} = $dither->[0];
			    $t{Y_DITHER} = $dither->[1];
			    $t{N_GAL}	 = 6000;
			    $t{N_STAR}	= 0;
			    $t{STELLAR} = false;
			    $t{CONST_PSF_SIZE} = 1;
			}
			
			Simgen::IO::tweak(\%t, \%a, \%paramFiles);
			Simgen::Engine::Run("shapelets","simages", \%software, \%a);
			
			{ # Back...
			    $t{GAMMA_1} = "";
			    $t{GAMMA_2} = "";
			    $a{"sh param file"} = "";
			    $t{D_CUT_LOWER} = "";
			    
			    $t{N_PIXELS} = "";
			    $t{X_DITHER} = "";
			    $t{Y_DITHER} = "";
			    $t{N_GAL}	= "";
			    $t{N_STAR}	= "";
			    $t{STELLAR} = "";
			    $t{CONST_PSF_SIZE} = "";
			}
			
			Simgen::Util::tock(@tick);
			
		    } # End galaxy field routine.
		} # End for each dither.
	    } # End for each gamma.

	    chomp(my @fileList = `ls -1 *_${t{N_RUN}}*`);
	    for my $thisFile (@fileList) {
		my $newFile = $thisFile;
		$newFile =~ s/_$t{N_RUN}//g;
		
		rename $thisFile, $newFile;
		run("rm ${t{DATA_OUT_HOME}}/$newFile");
		run("mv $newFile $t{DATA_OUT_HOME}");
	    }

	    run("mv $paramFiles{ 'sh param' }{file}	$t{DATA_OUT_HOME}		 ");
	    run("mv $paramFiles{ 'sh sex' }{file}	sh_$paramFiles{'sh sex'}{file}	 ");
	    run("mv sh_$paramFiles{ 'sh sex' }{file}	$t{DATA_OUT_HOME}		 ");
	    run("mv $paramFiles{ 'rrg param' }{file}	$t{DATA_OUT_HOME}		 ");
	    run("mv $paramFiles{ 'rrg sex' }{file}	rrg_$paramFiles{'rrg sex'}{file} ");
	    run("mv rrg_$paramFiles{ 'rrg sex' }{file} $t{DATA_OUT_HOME}		 ");
	    
	    $pxIndex++;
	} # End for each pixel scale.
    } # End for each random seed.

Simgen::Util::tock(@globalTick);

} elsif ( Simgen::Global::want2do eq "rrg" ) {
    ;
} else {
    message( "Don't how to do ", Simgen::Global::want2do );
    die;
}


# End simgen.pl.
