#!/usr/local/bin/perl

## simgen.pl
#
# A Perl script for generating simages a la R. Massey, and analyzing
# them for weak lensing with RRG.
#
# F. W. High
# 6/2004
##





use strict;





#
#
# FIXED GLOBAL VARIABLES AND INITIALIZERS.  DO NOT CHANGE.
#
#
my $true = 1;
my $false = 0;
srand (time ^ $$ ^ unpack "%L*", `ps axww | gzip`);





#
#
# Check commandline arguments.
#
#
if ( scalar @ARGV != 2 ) {

    die "Usage: simgen4.pl initDotPros? pixelScale\n";
}
my $initDotPros = @ARGV[0];
my @pixel_scale = @ARGV[1];





#
#
# OTHER GLOBAL VARIABLES.  DO CHANGE.
#
#

#
#
# Specify the IDL license availability.
#
#
my $licensesAvail = "";
my $licenses4me = "";
my $IDL_DIR = "";
my $institute = "Caltech";
if ( $institute eq "PDSF" ) {
    $licensesAvail = 120;
    $licenses4me = 70;
    $IDL_DIR = '/usr/local/pkg/IDL-6.0/idl_6.0'; # PDSF's path is set to this anyway.
} elsif ( $institute eq "Caltech" ) {
    $licensesAvail = 250;
    $licenses4me = 70;
    $IDL_DIR = '/usr/local/rsi/idl_6.0';
}

#
#
# Do you want simgen.pl to barf at you?	 How about logging all output
# to file?  How about print output directly to the screen?
#
#
my $verbose  = $true;
my $makeLog  = $false;
my $dryRun   = $false;
my $direct2screen = $true;

#
#
# Execute what?
#
#
my $simages = $true;
my $rrg = $false;
if ( $simages && $rrg ) {
    &message("You can't run simages and rrg at the same time.");
    die;
}


#
#
# Complete lists of tweaks and command line args.  These values will
# be initialized subsequently.	WARNING: THE SUBSEQUENT INITIALIZATION
# IS ORDER-DEPENDENT.  SHUFFLE WITH EXTREME CAUTION.
#
#

# t stands for tweak.
my %t = (
	 N_RUN		  => "",
	 TELESCOPE	  => "",
	 PIXEL_SCALE	  => "",
	 N_PIXELS	  => "",
	 X_DITHER	  => "",
	 Y_DITHER	  => "",
	 RANDOM_SEED	  => "",
	 NOISE_SEED	  => "",
	 N_GAL		  => "",
	 N_STAR		  => "",
	 STELLAR	  => "",
	 SIMGEN_HOME	  => "",
	 DIFF_LENGTH	  => "",
	 SNAP_PSF_NAME	  => "",
	 SNAP_PSF_FILE	  => "",
	 DATA_HOME	  => "",
	 DATA_IN_HOME	  => "",
	 DATA_OUT_HOME	  => "",
	 SEX_PATH	  => "",
	 GAMMA_1          => "",
	 GAMMA_2          => "",
	 INPUT_CATALOGUE  => "",
	 MULT             => "",
	 MIN_RAD          => "",
	 SN               => "",
	 D_CUT_LOWER      => "",
	 D_CUT_UPPER      => "",
	 CONST_PSF_SIZE   => ""
	 );

# a stands for [commandline] argument.
my %a = (
	 "sh param file"  => "",
	 "rrg param file" => "",
	 );



#
#
# This is usefull for parallel execution.
#
#
{
    chomp(my $hostname = `hostname`);
    $t{N_RUN} = "host-${hostname}_pid-$$";
}
my $nRun = $t{N_RUN};

#
# Specify the pixel scales.
#
# my @pixel_scale = [
#		   0.04,
#		   0.05,
#		   0.06,
#		   0.07,
#		   0.08,
#		   0.09,
#		   0.10,
#		   0.11,
#		   0.12,
#		   0.13,
#		   0.14
#		   ];


#
#
# Diffusion length of the CCD in microns rms.
#
#
$t{DIFF_LENGTH} = 4.0;

#
#
# Specify the input catalog.
#
#
$t{INPUT_CATALOGUE} = "F";

#
#
# Specify the telescope.
#
#
$t{TELESCOPE} = "SNAP";

#
#
# Specify the SNAP telescope's PSF name.
#
#
$t{SNAP_PSF_NAME} = "SNAP_fft11";
if ( $t{SNAP_PSF_NAME} eq "SNAP_PSF" ) {

    $t{SNAP_PSF_FILE} = "SNAP_psf_800nm_01radians_back.fits";

} elsif ( $t{SNAP_PSF_NAME} eq "SNAP_fft11" ) {

    $t{SNAP_PSF_FILE} = "fft11.fits";

} else {

    &message("Invalid SNAP PSF name.");
    die;
}


#
#
# Determine the proportionate number of pixels.
#
#
my @n_pixels = (0 .. $#pixel_scale);
for my $i (0 .. $#pixel_scale) {
    $n_pixels[$i] = int( 4096 * 0.04 / $pixel_scale[$i] );
}

#
#
# Specify the home directories.
#
#
$t{SIMGEN_HOME} = "/home/elen/fwh/simgen";
$t{DATA_HOME}	= "/scr/rjm/fwh/data";
$t{SEX_PATH}	= "/usr/local/optical/sextractor/bin/sex";

#
#
# Initialize the input directory.
#
#
$t{DATA_IN_HOME} = $t{DATA_HOME} . "/in";

#
#
# Initialize the output directory.
#
#
#{
#    chomp(my $date = `date +%Y-%m-%d`);
#    $t{DATA_OUT_HOME} = $t{DATA_HOME} . "/out/${date}";
#    my @existingDirs = `ls -1d ${t{DATA_OUT_HOME}}*`;
##    my $dirSuffix = "${t{N_RUN}}";
#    my $n = $#existingDirs + 2;
#    my $dirSuffix = "_no.$n";
#    $t{DATA_OUT_HOME} = "${t{DATA_OUT_HOME}}_$dirSuffix";
#}
# Actually, let's do it by hand.
#my $dirSuffix = "_no.2";
my $dirSuffix = "_test";
if ( $t{SNAP_PSF_NAME} eq "SNAP_PSF" ) {

    $t{DATA_OUT_HOME} = $t{DATA_HOME} . "/out/snap_psf$dirSuffix";

} elsif ( $t{SNAP_PSF_NAME} eq "SNAP_fft11" ) {

    $t{DATA_OUT_HOME} = $t{DATA_HOME} . "/out/snap_fft11$dirSuffix";

} else {

    &message("Error determining data out directory.");
    die;
}
unless (mkdir $t{DATA_OUT_HOME}) {

    if ( -d $t{DATA_OUT_HOME} ) {
	&message("Directory $t{DATA_OUT_HOME} already exists.");
    } else {
	&message("Error making output directory $t{DATA_OUT_HOME}.");
	die;
    }
}

#
#
# Set some of the RRG parameter values.
#
#
$t{MULT} = 2;
$t{MIN_RAD} = 3.5;
$t{SN} = 10;
$t{D_CUT_UPPER} = 2000;

#
#
# Indicate the shear scheme.  For each pixel scale, there will one
# simage for each of these sets of shear.
#
#
my @gamma = (
#	     [	0.000, -0.050 ],
#	     [	0.000, -0.025 ],
	     [	0.000,	0.000 ],
#	     [	0.000,	0.025 ],
#	     [	0.000,	0.050 ],
#	     [ -0.050,	0.000 ],
#	     [ -0.025,	0.000 ],
##	     [	0.000,	0.000 ],
#	     [	0.025,	0.000 ],
#	     [	0.050,	0.000 ]
	     );

#
#
# Indicate the dither scheme.
#
#
my @dither = (
	      [ 0.0, 0.0 ],
#	      [ 0.5, 0.0 ],
#	      [ 0.5, 0.5 ],
#	      [ 0.0, 0.5 ]
	      );

#
#
# Here are all implemented software packages.
#
#
my %software = (
		shapelets => {
		    simages => {
			exec => "${t{SIMGEN_HOME}}/simages/generate_simage",
			args => [ "sh param file" ],
		    },
		},
		rrg => {
		    psf => {
			exec => "${t{SIMGEN_HOME}}/rrg/rrg_get_psf",
			args => [ "rrg param file" ],
		    },
		    shear => {
			exec => "${t{SIMGEN_HOME}}/rrg/rrg_get_shear",
			args => [ "rrg param file" ],
		    },
		},
		); # End software hash.

#
#
# Here are all parameter files used by any software package.
#
#
sub initParamFiles {
    my %paramFiles = (
		      "sh param" => {
			  "default file" => "${t{SIMGEN_HOME}}/simages/telescope_parameters.simgen",
			  file	  => "${t{SIMGEN_HOME}}/simages/telescope_parameters_$nRun",
			  tweaks => [ "PIXEL_SCALE",
				      "TELESCOPE",
				      "N_PIXELS",
				      "RANDOM_SEED",
				      "NOISE_SEED",
				      "N_RUN",
				      "N_GAL",
				      "N_STAR",
				      "STELLAR",
				      "DIFF_LENGTH",
				      "SNAP_PSF_NAME",
				      "GAMMA_1",
				      "GAMMA_2",
				      "INPUT_CATALOGUE",
				      "X_DITHER",
				      "Y_DITHER",
				      "CONST_PSF_SIZE" ],
		      },
		      "sh sex"	=> {
			  "default file" => "${t{SIMGEN_HOME}}/shapelets/shextractor/config/${t{TELESCOPE}}.sex.simgen",
			  file	  => "${t{SIMGEN_HOME}}/shapelets/shextractor/config/${t{TELESCOPE}}_${nRun}.sex",
			  tweaks => [ "PIXEL_SCALE"],
		      },
		      "sh paths" => {
			  "default file" => "${t{SIMGEN_HOME}}/shapelets/shapelets_paths.pro.simgen",
			  file => "${t{SIMGEN_HOME}}/shapelets/shapelets_paths.pro",
			  tweaks => [ "SIMGEN_HOME",
				      "DATA_IN_HOME",
				      "SEX_PATH",
				      "SNAP_PSF_FILE" ],
		      },

		      "rrg param" => {
			  "default file" => "${t{SIMGEN_HOME}}/rrg/rrg_parameters.simgen",
			  file	  => "${t{SIMGEN_HOME}}/rrg/rrg_parameters_$nRun",
			  tweaks => [ "PIXEL_SCALE",
				      "TELESCOPE",
				      "N_PIXELS",
				      "RANDOM_SEED",
				      "NOISE_SEED",
				      "N_RUN",
				      "N_GAL",
				      "N_STAR",
				      "STELLAR",
				      "DIFF_LENGTH",
				      "SNAP_PSF_NAME",
				      "GAMMA_1",
				      "GAMMA_2",
				      "INPUT_CATALOGUE",
				      "X_DITHER",
				      "Y_DITHER",
				      "MULT",
				      "MIN_RAD",
				      "SN",
				      "D_CUT_LOWER",
				      "D_CUT_UPPER" ],
		      },
		      "rrg sex" => {
			  "default file" => "${t{SIMGEN_HOME}}/rrg/sextractor/${t{TELESCOPE}}.sex.simgen",
			  file	  => "${t{SIMGEN_HOME}}/rrg/sextractor/${t{TELESCOPE}}_${nRun}.sex",
			  tweaks => [ "PIXEL_SCALE" ],
		      },
		      "rrg paths" => {
			  "default file" => "${t{SIMGEN_HOME}}/rrg/rrg_paths.pro.simgen",
			  file => "${t{SIMGEN_HOME}}/rrg/rrg_paths.pro",
			  tweaks => [ "SIMGEN_HOME",
				      "DATA_IN_HOME",
				      "SEX_PATH",
				      "SNAP_PSF_FILE" ],
		      }

		      ); # End paramFiles hash.
}
my %paramFiles = &initParamFiles();



#
#
# BEGIN MAIN PROGRAM.
#
#

my $logFile = "";


#
#
# BEGIN MAIN NESTED LOOPS.
#
#

#
# Begin simages.
#
if ($simages) {

    my @globalTick = &tick("simages global stopwatch");

    chdir $t{DATA_IN_HOME};
    &run("pwd");
    
    if ($makeLog) {
	
	$logFile = openLog("$t{DATA_OUT_HOME}/simgen_simages_${t{N_RUN}}.log");
	initLog(%paramFiles);
    }
    
    if ( $initDotPros ) {
	&initweak("sh paths");
	&initweak("rrg paths");
    }

    my $simagesCount = 0;
    for ( $t{RANDOM_SEED} = 1; $t{RANDOM_SEED} < 2; $t{RANDOM_SEED}++ ) {

	my $pxIndex = 0;
	for my $pixel_scale ( @pixel_scale ) {
	    $t{PIXEL_SCALE} = $pixel_scale;
	    
	    my $needStarField = $true;

	    &run("rm *_${t{N_RUN}}*");

	    for my $gamma ( @gamma ) {

		for my $dither ( @dither ) {
		    
		    $simagesCount++;
		    $t{NOISE_SEED} = $simagesCount;

		    if ($needStarField) { # Begin star field routine.

			&message("Simulating a globular cluster using simages.");

			my @tick = &tick("local stopwatch");

			{ # ... and forth.
			    $t{GAMMA_1} = 0;
			    $t{GAMMA_2} = 0;
			    $a{"sh param file"} = $paramFiles{"sh param"}{file};
			    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
			    $t{D_CUT_LOWER} = 1.2 * sqrt($factor * $factor + 0.36);
			    
			    $t{N_PIXELS} = $n_pixels[$pxIndex];
			    $t{X_DITHER} = @$dither[0];
			    $t{Y_DITHER} = @$dither[1];
			    $t{N_GAL}	 = 0;
			    $t{N_STAR}	= 372;
			    $t{STELLAR} = $true;
			    $t{CONST_PSF_SIZE} = 1;
			}

			&tweak();
			&execute("shapelets","simages");

			{ # Back...

			    $t{GAMMA_1} = "";
			    $t{GAMMA_2} = "";
			    $a{"sh param file"} = "";
			    $t{D_CUT_LOWER} = "";
			    
			    $t{N_PIXELS} = "";
			    $t{X_DITHER} = "";
			    $t{Y_DITHER} = "";
			    $t{N_GAL}	= "";
			    $t{N_STAR}	= "";
			    $t{STELLAR} = "";
			    $t{CONST_PSF_SIZE} = "";
			}

			&tock(@tick);
			
			$needStarField = $false;
		    } # End star field routine.

		    if ($true) { # Begin galaxy field routine.

			&message("Simulating a galaxy field using simages.");
			
			my @tick = &tick("local stopwatch");
			
			{ # ...and forth.
			    $t{GAMMA_1} = @$gamma[0];
			    $t{GAMMA_2} = @$gamma[1];
			    $a{"sh param file"} = $paramFiles{"sh param"}{file};
			    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
			    $t{D_CUT_LOWER} = 1.2 * sqrt($factor * $factor + 0.36);
			    
			    $t{N_PIXELS} = $n_pixels[$pxIndex];
			    $t{X_DITHER} = @$dither[0];
			    $t{Y_DITHER} = @$dither[1];
			    $t{N_GAL}	 = 6000;
			    $t{N_STAR}	= 0;
			    $t{STELLAR} = $false;
			    $t{CONST_PSF_SIZE} = 1;
			}
			
			&tweak();
			&execute("shapelets","simages");
			
			{ # Back...
			    $t{GAMMA_1} = "";
			    $t{GAMMA_2} = "";
			    $a{"sh param file"} = "";
			    $t{D_CUT_LOWER} = "";
			    
			    $t{N_PIXELS} = "";
			    $t{X_DITHER} = "";
			    $t{Y_DITHER} = "";
			    $t{N_GAL}	= "";
			    $t{N_STAR}	= "";
			    $t{STELLAR} = "";
			    $t{CONST_PSF_SIZE} = "";
			}
			
			&tock(@tick);
			
		    } # End galaxy field routine.
		} # End for each dither.
	    } # End for each gamma.

	    chomp(my @fileList = `ls -1 *_${t{N_RUN}}*`);
	    for my $thisFile (@fileList) {
		my $newFile = $thisFile;
		$newFile =~ s/_$t{N_RUN}//g;
		
		rename $thisFile, $newFile;
		&run("rm ${t{DATA_OUT_HOME}}/$newFile");
		&run("mv $newFile $t{DATA_OUT_HOME}");
	    }

	    &run("mv $paramFiles{ 'sh param' }{file}	$t{DATA_OUT_HOME}		 ");
	    &run("mv $paramFiles{ 'sh sex' }{file}	sh_$paramFiles{'sh sex'}{file}	 ");
	    &run("mv sh_$paramFiles{ 'sh sex' }{file}	$t{DATA_OUT_HOME}		 ");
	    &run("mv $paramFiles{ 'rrg param' }{file}	$t{DATA_OUT_HOME}		 ");
	    &run("mv $paramFiles{ 'rrg sex' }{file}	rrg_$paramFiles{'rrg sex'}{file} ");
	    &run("mv rrg_$paramFiles{ 'rrg sex' }{file} $t{DATA_OUT_HOME}		 ");
	    
	    $pxIndex++;
	} # End for each pixel scale.
    } # End for each random seed.

    &tock(@globalTick);

    close(LOG) if $makeLog;

} # End if simages.
#
# End simages.
#



#
# Begin RRG.
#
if ($rrg) {

    my @globalTick = &tick("rrg global stopwatch");

#
#
# Initialize the input directory.
#
#
    $t{DATA_IN_HOME} = $t{DATA_OUT_HOME};
    chdir $t{DATA_IN_HOME};
    &run("pwd");
    
#
#
# Initialize the output directory.
#
#
#    {
# 	$t{DATA_OUT_HOME} = $t{DATA_OUT_HOME} . "/rrg";
# 	my @existingDirs = `ls -1d ${t{DATA_OUT_HOME}}*`;
# 	my $n = $#existingDirs + 2;
# 	my $dirSuffix = "no.$n";
# 	$t{DATA_OUT_HOME} = "${t{DATA_OUT_HOME}}_$dirSuffix";
#    }
    # Actually, let's do it by hand.
    $t{DATA_OUT_HOME} = $t{DATA_OUT_HOME} . "/rrg_no.4";
    unless (mkdir $t{DATA_OUT_HOME}) {

	if ( -d $t{DATA_OUT_HOME} ) {
	    &message("Directory $t{DATA_OUT_HOME} already exists.");
	} else {
	    &message("Error making output directory $t{DATA_OUT_HOME}.");
	    die;
	}
    }

    if ($makeLog) {
	
	$logFile = openLog("$t{DATA_OUT_HOME}/simgen_rrg_${nRun}.log");
	initLog(%paramFiles);
    }

#
#
# The N_RUN idl variable is now obselete?
#
#
    $t{N_RUN} = "";

    %paramFiles = &initParamFiles();

# The following value is irrelevant, I think.  It _must_ be
# initialized, though. 
    $t{RANDOM_SEED} = 1;

    $t{X_DITHER} = 0;
    $t{Y_DITHER} = 0;

    if ( $initDotPros ) {
	&initweak("sh paths");
	&initweak("rrg paths");
    }

# These have been empirically determined using sqrt((Ixx + Iyy)/2) of
# the PSF.
my @factor = (
	      0.886096,
	      0.768605,
	      0.692932,
	      0.668683,
	      0.653262,
	      0.620874,
	      0.602347,
	      0.592070,
	      0.569379,
	      0.596764,
	      0.580939
	      );

    my $rrgCount = 0;
    my $pxIndex = 0;
    for my $pixel_scale ( @pixel_scale ) {
	$t{PIXEL_SCALE} = $pixel_scale;
	
	my $needStarField = $true;

	for my $gamma ( @gamma ) {
	    
	    $rrgCount++;
	    $t{NOISE_SEED} = $rrgCount;

	    if ($needStarField) { # Begin star field routine.

		&message("Get PSF from globular cluster using rrg.");

		my @tick = &tick("local stopwatch");

		{ # ... and forth.
		    $t{GAMMA_1} = 0;
		    $t{GAMMA_2} = 0;
		    $a{"rrg param file"} = $paramFiles{"rrg param"}{file};
#		    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
		    my $fIndex = $t{PIXEL_SCALE} * 100 - 4;
		    $t{D_CUT_LOWER} = 2.0 * sqrt($factor[$fIndex] * $factor[$fIndex] + 0.36);
		    
		    $t{N_PIXELS} = $n_pixels[$pxIndex];
		    $t{N_GAL}	 = 0;
		    $t{N_STAR}	= 372;
		    $t{STELLAR} = $true;
		    $t{CONST_PSF_SIZE} = 1;
		}

		&tweak();
		&execute("rrg","psf");

		{ # Back...
		    $t{GAMMA_1} = '';
		    $t{GAMMA_2} = '';
		    $a{"rrg param file"} = "";
		    $t{D_CUT_LOWER} = "";
		    
		    $t{N_PIXELS} = "";
		    $t{N_GAL}	= "";
		    $t{N_STAR}	= "";
		    $t{STELLAR} = "";
		    $t{CONST_PSF_SIZE} = 0;
		}

		&tock(@tick);
		
		$needStarField = $false;
	    } # End star field routine.

	    if ($true) { # Begin galaxy field routine.
		
		&message("Get shear from galaxy field using rrg.");

		my @tick = &tick("local stopwatch");
		
		{ # ...and forth.
		    $t{GAMMA_1} = @$gamma[0];
		    $t{GAMMA_2} = @$gamma[1];
		    $a{"rrg param file"} = $paramFiles{"rrg param"}{file};
#		    my $factor = 2.0 * 7.14604 * 0.0096 / $t{PIXEL_SCALE};
		    my $fIndex = $t{PIXEL_SCALE} * 100 - 4;
		    $t{D_CUT_LOWER} = 2.0 * sqrt($factor[$fIndex] * $factor[$fIndex] + 0.36);# * ($rrgCount + 3) / ($rrgCount + 4);
		    
		    $t{N_PIXELS} = $n_pixels[$pxIndex];
		    $t{N_GAL}	 = 6000;
		    $t{N_STAR}	= 0;
		    $t{STELLAR} = $false;
		    $t{CONST_PSF_SIZE} = 1;
		}
		
		&tweak();
		&execute("rrg","shear");
		
		{ # Back...
		    $t{GAMMA_1} = "";
		    $t{GAMMA_2} = "";
		    $a{"rrg param file"} = "";
		    $t{D_CUT_LOWER} = "";
		    
		    $t{N_PIXELS} = "";	    
		    $t{N_GAL}	= "";
		    $t{N_STAR}	= "";
		    $t{STELLAR} = "";
		    $t{CONST_PSF_SIZE} = 0;
		}
		
		&tock(@tick);
		
	    } # End galaxy field routine.
	} # End for each gamma.

	&run("mv $paramFiles{ 'sh param' }{file}   $t{DATA_OUT_HOME}		    ");
	&run("mv $paramFiles{ 'sh sex' }{file}	   sh_$paramFiles{'sh sex'}{file}   ");
	&run("mv sh_$paramFiles{ 'sh sex' }{file}  $t{DATA_OUT_HOME}		    ");
	&run("mv $paramFiles{ 'rrg param' }{file}   $t{DATA_OUT_HOME}		     ");
	&run("mv $paramFiles{ 'rrg sex' }{file}	    rrg_$paramFiles{'rrg sex'}{file} ");
	&run("mv rrg_$paramFiles{ 'rrg sex' }{file} $t{DATA_OUT_HOME}		     ");

	my $px = 100 * $t{PIXEL_SCALE};
	if ($px < 10) {
	    &run("mv *p0.0${px}*.coeff $t{DATA_OUT_HOME}");
	    &run("mv *p0.0${px}*.shear $t{DATA_OUT_HOME}");
	    &run("mv *p0.0${px}*.rrg_out $t{DATA_OUT_HOME}");
	    &run("mv *p0.0${px}*.cat $t{DATA_OUT_HOME}");
	    &run("mv *p0.0${px}*.mom* $t{DATA_OUT_HOME}");
	} else {
	    &run("mv *p0.${px}*.coeff $t{DATA_OUT_HOME}");
	    &run("mv *p0.${px}*.shear $t{DATA_OUT_HOME}");
	    &run("mv *p0.${px}*.rrg_out $t{DATA_OUT_HOME}");
	    &run("mv *p0.${px}*.cat $t{DATA_OUT_HOME}");
	    &run("mv *p0.${px}*.mom* $t{DATA_OUT_HOME}");
	}
	$pxIndex++;
    } # End for each pixel scale.

    &tock(@globalTick);

    close(LOG) if $makeLog;

} # End if rrg.
#
# End RRG.
#

#
#
# END MAIN NESTED LOOPS.
#
#




#
#
# END MAIN PROGRAM.
#
#





#
#
# BEGIN SUBROUTINES.
#
#

#
#
# Open the log file.
#
#
sub openLog {

    my $logFile = $_[0];
    
#
# Prepare log file.
#
    if (-f $logFile) {
	my @existingLogs = `ls -1 ${logFile}*`;
	my $suffix = $#existingLogs + 1;
	rename $logFile, $logFile . ".$suffix.old";
    }
    open(LOG,">>$logFile") || die "simgen: Can't open $logFile for logging output.\n";

    $logFile;
} # End sub openLog.


#
#
# Initialize the log file.
#
#
sub initLog {

    chomp(my $when = `date`);
    print LOG "\n\n*** NEW SESSION: $when ***\n\n";
    print LOG "simgen: Following are the runtime parameters used to get the subsequent output.\n";
    print LOG "simgen: They come from the script you fed into the executable.\n";
    
    for my $key ( keys %paramFiles ) {

	if ( $paramFiles{$key}{"default file"} ) {

	    print LOG "simgen: Begin default parameter file $paramFiles{$key}{'default file'}.\n\n";
	    unless ( open( IN,$paramFiles{$key}{"default file"} ) ) {
		&message("Can't open $paramFiles{$key}{'default file'} for reading");
		die;
	    }

	    while (<IN>) {
		print LOG $_;
	    }

	    print LOG "\nsimgen: End default parameter file $paramFiles{$key}{'default file'}.\n";
	    close(IN);
	}
    }
} # End sub initLog.


#
#
# Run 
#
#
sub run {

    &message("Executing @_\n");

    if ($direct2screen) {
# BEGIN TEMP
	system(@_);
# END TEMP
    } else {
	my @out = `@_ 2>&1`;
	if ($makeLog) {
	    print LOG "\n";
	    print LOG @out;
	    print LOG "\n";
	}
	if ($verbose) {
	    print "\n";
	    print @out;
	    print "\n";
	}
    }
} # End sub run.


#
#
# Print messages to various places.
#
#
sub message {

    if ($verbose) {
	print "simgen: @_\n";
    }
    if ($makeLog) {
	print LOG "simgen: @_\n";
    }
} # End sub message.


#
#
# Verify that at least one simgen variable exists in a file.
#
#
sub verifyFile {
    my $initialString;
    $initialString = $_[0];
    my $paramFile;
    $paramFile = $_[1];

    if (&existsInFile($initialString,$paramFile)) {
#	&message("File $paramFile");
#	&message("is _probably_ properly initialized with $initialString");
    } else {
	&message("File $paramFile not initialized properly.");
	&message("Please put $initialString somewhere in it.");
	die;
    }

    unless (&existsInFile($initialString,$paramFile)) {
	die;
    }
} # End sub verifyFile.


#
#
# Replace all instances of $thisString with $nextString in file $inFile.
#
#
sub replaceInFile {

    my $thisString;
    $thisString = $_[0];
    my $nextString;
    $nextString = $_[1];
    my $inFile;
    $inFile = $_[2];

    &verifyFile($thisString,$inFile);

    my $tmpFile = "${nRun}.tmp";
    my $inline;

# Open the desired files for input and output.
    unless (open(IN,$inFile)) {
	&message("Cannot open file $inFile for reading.");
	die;
    }
    unless (open(OUT,">$tmpFile")) {
	&message("Cannot open temporary file $tmpFile for writing.");
	die;
    }

    &message("Setting $thisString to $nextString...");
    &message("...in $inFile");
# Replace (squeeze) each instance of $initialString with the first section number in each line,
# then ouput all lines to OUT file.
    while ($inline = <IN>) {
	$inline =~ s/$thisString/$nextString/g;
	print OUT $inline;
    }
# Close all open files.
    close(IN);
    close(OUT);
# Swap dummy $tmpFile to original $inFile.
#    unless (rename($tmpFile,$inFile)) {
#	&message("Can't rename $tmpFile to $inFile.");
#	die;
#    }
    `mv $tmpFile $inFile`;
} # End sub replaceInFile.


#
#
# Check for at least one instance of $stringToCheck in $file.
#
#
sub existsInFile {

    my $stringToCheck;
    $stringToCheck = $_[0];
    my $file;
    $file = $_[1];

    my $goodFile;

    unless (open(IN,$file)) {
	&message("Can't open $file for reading");
	die;
    }
    $goodFile = $false;
    while (<IN>) {
	if (/$stringToCheck/) {
	    $goodFile = $true;
	}
    }
    close(IN);

# Return this value.
    $goodFile;
} # End sub existsInFile.


#
#
# Convert seconds to hours:minutes:seconds.
#
#
sub sec2hms {
    my $runtime = $_[0];

    my $hours = $runtime / 3600;
    $hours = int($hours);
    my $remainder = $runtime % 3600;
    my $minutes = $remainder / 60;
    $minutes = int($minutes);
    my $seconds = ($remainder % 60) + (($runtime * 100) % 100) / 100;

    ($hours, $minutes, $seconds);
} # End sub sec2hms.


#
#
# Tweak the various tweaks in almost all the parameter files.
#
#
sub tweak {

    &printVals();

    for my $title ( keys %paramFiles ) {

	&message("Skipping $title initialization.") and next if ( $title eq "sh paths" || $title eq "rrg paths" );

	&initweak("$title");
    }
} # End sub tweak.

#
#
# Initialize only the specified parameter file.
#
#
sub initweak {

    my $title = $_[0];

    my $tmpFile = "$paramFiles{$title}{file}_${nRun}.tmp";
    &run("cp $paramFiles{$title}{'default file'} $tmpFile");

    for my $tweak ( @{$paramFiles{$title}{tweaks}} ) {

	if ( $t{$tweak} eq "" && $tweak ne "N_RUN" && !$rrg ) {
	    &message("Tweak $tweak hasn't been set *or* $tweak doesn't exist!");
	    die;
	}
	&replaceInFile("${tweak}.SIMGEN","$t{$tweak}",$tmpFile);
    }
    
    &run("mv $tmpFile $paramFiles{$title}{file}");
    
} # End sub initweak.

#
#
# Execute the desired software, as indicated in @toDo.
#
#
sub execute {

    my @toDo = @_;
    
    if ( scalar(@toDo) % 2 != 0 ) {
	
	&message("List of commands in 'execute' must have even no. of elements.");
	die;
    }
    
    for my $j ( 1 .. scalar(@toDo)/2 ) {

	my $method  = $toDo[ $j*2 - 2 ];
	my $routine = $toDo[ $j*2 - 1 ];

	if ( !scalar($software{$method}) ) {

	    &message("Method '$method' unknown.");
	    die;
	}
	if ( !scalar(%{$software{$method}{$routine}}) ) {

	    &message("Routine '$routine' unkown for method '$method'.");
	    die;
	}
    }
    
    for my $j ( 1 .. scalar(@toDo)/2 ) {

	my $method  = $toDo[ $j*2 - 2 ];
	my $routine = $toDo[ $j*2 - 1 ];

	my $argVal = "";
	my $argList = "";

	for my $argName ( @{$software{$method}{$routine}{args}} ) {
	    
	    $argVal = $a{$argName};
	    
	    if ( $argVal eq "" ) {
		
		&message("Arg $argVal doesn't appear in master arg list *or* wasn't initialized!");
		die;
	    }
	    if ($argName eq "gamma") {

		if (@$argVal[0] eq "" || @$argVal[1] eq "" ) {

		    &message("Gamma not initialized properly.");
		    die;
		}

		$argVal = "@$argVal[0] @$argVal[1]";
	    }

	    $argList = "$argList $argVal";
	}

	my $exec = $software{$method}{$routine}{exec};

	if ($dryRun) {
	    &message("Executing $exec $argList");
	} else {
	    &wait4license();
	    &run("$exec $argList");

# 	    if ( $institute eq "PDSF" ) {
# 		my $scriptFilename = "simsub.csh";
# 		open(SCRIPT, "> $scriptFilename ") || die("can't open logfile:	$!");
# 		print SCRIPT "#!/bin/csh\n";
# 		print SCRIPT "$exec $argList\n";
# 		print SCRIPT "\n" && close(SCRIPT);
# 		&run("chmod +x $scriptFilename");
# 		&run("touch ${scriptFilename}.bsub_out");
# 		&run("bsub -q long -o ${scriptFilename}.bsub_out < $scriptFilename");
# 		&run("rm $scriptFilename");
# 		exit 0;
# 	    } elsif ( $institute eq "Caltech" ) {
# 		my $n = 1;
# 		&wait4license();
# 		&run("ssh -x kishar$n '$exec $argList'");
# 	    } else {
# 		&message("Unable to run becuase institute is unknown.");
# 	    }

	}
    }
} # End sub execute.

#
#
# Wait for an available IDL license, and don't hog them.
#
#
sub wait4license {

    my $ready = $false;
    while (!$ready) {
	chomp(my $n_6  = int `$IDL_DIR/bin/lmstat -A | grep  '6 licenses' | wc -l`);
	chomp(my $n_10 = int `$IDL_DIR/bin/lmstat -A | grep '10 licenses' | wc -l`);
	my $inUse = 10*$n_10 + 6*$n_6;
	chomp($n_6  = int `$IDL_DIR/bin/lmstat -A | grep  '6 licenses' | grep fwh | wc -l`);
	chomp($n_10 = int `$IDL_DIR/bin/lmstat -A | grep '10 licenses' | grep fwh | wc -l`);
	my $inUseByMe = 10*$n_10 + 6*$n_6;
	
	&message("$inUse out of $licensesAvail IDL licenses in use, $inUseByMe by me.");

	if ($inUse >= int($licensesAvail*5.0/6.0) || $inUseByMe >= $licenses4me) {
	    chomp(my $datum = `date`);
	    &message("IDL unavailable at ${datum}.");
	    sleep 600;
	    $ready = $false;
	} else {
	    $ready = $true;
	}
    }
}



#
#
# Subroutines tick and tock together are kind of a stopwatch.
# &tock(@tick) tells you the elapsed time since @tick =
# &tick("unique stopwatch id").
#
#
sub tick {
    
    my $id = $_[0];

    chomp(my $date = `date`);
    &message("Starting stopwatch '$id' on ${date}.");

    chomp( my $tick = `date +%s` );

    my @tick = ($tick, $id);
} # End sub tick.

sub tock {

    my $tick = $_[0];
    my $id = $_[1];

    chomp(my $date = `date`);
    &message("Stopping stopwatch '$id' on ${date}.");

    chomp( my $tock = `date +%s` );

    my $runtime = $tock - $tick;
    (my $hours, my $minutes, my $seconds) = &sec2hms($runtime);
    &message("Runtime = $runtime sec, or $hours:$minutes:$seconds hr:min:sec");
} # End sub tock.

#
#
# Print the values of the main hashes.
#
#
sub printVals {

    &message("Tweaks...");
    for my $key ( sort keys %t ) {
	&message( "\t$key \t= $t{$key}" );
    }

    &message("Commandline arguments...");
    for my $key ( sort keys %a ) {
	if ( $key eq "gamma" ) {
	    &message( "\t$key \t= [ $a{$key}[0], $a{$key}[1] ]" );
	} else {
	    &message( "\t$key \t= $a{$key}" );
	}
    }
#    for my $key1 ( sort keys %paramFiles ) {
#	for my $key2 ( keys %{$paramFiles{$key1}} ) {
#	    &message( "$key1 $key2 = $paramFiles{$key1}{$key2}" );
#	}
#    }
} # End sub printVals.

#
#
# beepMsg
#
#
sub bMessage {

    my @message = @_;

    unless ($#message eq -1) {

	chomp( $message[$#message] );
	$message[$#message] = $message[$#message] . "\a\n";
    }
    
    &message(@message);

}

#
#
# END SUBROUTINES.
#
#





#
#
# BEGIN SCRATCH AREA.
#
#

# This is the order of execution.
#my @software = ( "shapelets", "rrg" );

# sub execute {
#     my $i = $_[0];
#     my $shearOrPsf = $_[1];
#     unless ($shearOrPsf eq "shear" || $shearOrPsf eq "psf") {
#	&message("Set second arg of 'execute' to 'shear' or 'psf'.");
#	die;
#     }
# #    for my $method ( keys %software ) { # Hash keys are returned unordered!
# #    for my $method ( ( "shapelets", "rrg" ) ) {
#     for my $method( @software ) {
#	for my $routine ( keys %{$software{$method}} ) {
#	    my $argList = "";
#	    my $argVal = "";
#	    for my $argName ( @{$software{$method}{$routine}{args}} ) {
#		$argVal = $a{$argName};
#		if ( $argVal eq "" ) {
#		    &message("Arg $argVal doesn't appear in master arg list *or* wasn't initialized!");
#		    die;
#		}
#		if ($argName eq "gamma") {
#		    if (@$argVal[0] eq "" || @$argVal[1] eq "" ) {
#			&message("Gamma not initialized properly.");
#			die;
#		    }
#		    $argVal = "@$argVal[0] @$argVal[1]";
#		}
#		$argList = "$argList $argVal";
#	    }
#	    my $exec = $software{$method}{$routine}{exec};
#	    if ($routine eq $shearOrPsf || ($routine ne "psf" && $routine ne "shear")) {
#		&run("$exec $argList");
#	    }
#	}
#     }
# }

#
#
# END SCRATCH AREA.
#
#


#
#
# END simgen.pl.
#
#
