#!/usr/bin/perl

use lib "$ENV{SIMGEN}";

use strict;
use warnings;
die "Please set the SIMGEN environment variable" unless $ENV{SIMGEN};
use lib "$ENV{SIMGEN}";
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::IO;
use Simgen::Init;
use Simgen::Util;

if ( scalar @ARGV < 2 ) {
    die "ERROR\n";
}

my $what = $ARGV[0];
my $algo = $ARGV[1];
my $schemeName = $ARGV[2];
{
    my $thisScheme = Simgen::Global::Scheme( $schemeName );
    die "ERROR\n" if $thisScheme eq "ERROR";
}

# Initialize variables from global libraries.
my %t = Simgen::Init::Tweaks;
my %paramFiles = Simgen::Init::ParamFiles( \%t );
my %software = Simgen::Init::Software( \%t );
my $scheme = Simgen::Global::Scheme( $schemeName );
my %scheme = %$scheme;
my @pixel_scale = @{$scheme{pixel_scale}};
my @n_pixels = Simgen::Init::N_Pixels( \@pixel_scale );
my @gamma = @{$scheme{gamma}};
my @dither = @{$scheme{dither}};

# "param" variables.
my $run_arg;
# "run" variables.
my $getTotal = 0;
my $getPrefix = 0;
my $psf_arg;
my $stellar_arg;
my $pixel_scale_arg;
my @gamma_arg;
my @dither_arg;

die "ERROR\n" unless ( $algo eq "shapelets" || $algo eq "rrg" || $algo eq "drizzle" );

if ( $what eq "param" ) {

    die "Not working yet";

    die "ERROR\n" unless scalar @ARGV == 4;
    $run_arg = $ARGV[1];

} elsif ( $what eq "run" ) {

    if ( $algo eq "shapelets" || $algo eq "rrg" ) {
	if ( scalar @ARGV == 3 ) {
	    $getTotal = 1;
	} elsif ( scalar @ARGV == 10 ) {
	    $psf_arg = $ARGV[3];
	    $stellar_arg = $ARGV[4];
	    $pixel_scale_arg = $ARGV[5];
	    @gamma_arg = @ARGV[6,7];
	    @dither_arg = @ARGV[8,9];
	} else {
	    die "ERROR\n";
	}
    } elsif ( $algo eq "drizzle" ) {
	if ( scalar @ARGV == 3 ) {
	    $getTotal = 1;
	} elsif ( scalar @ARGV == 8 ) {
	    $psf_arg = $ARGV[3];
	    $stellar_arg = $ARGV[4];
	    $pixel_scale_arg = $ARGV[5];
	    @gamma_arg = @ARGV[6,7];
	} else {
	    die "ERROR\n";
	}
    } else {
	die "ERROR\n";
    }

} elsif ( $what eq "total" ) {

    $getTotal = 1;

} elsif ( $what eq "prefix" ) {

    $getPrefix = 1;
    print Simgen::Util::rootFilename(0,4,$algo,$getPrefix) . "\n";
    exit 0;

} else {

    die "ERROR\n";

}

my $doBoolean = 1;
$doBoolean = 0 if ( $getTotal || $getPrefix );
my $run_count = 0;
my $drz_count = 0;
for my $pixel_scale ( @pixel_scale ) {
    my $stellar = 1;
    for my $gamma ( @gamma ) {
	if ( $stellar ) {
	    # Stellar field.  The gamma_i commandline inputs don't
	    # matter here.
	    #my $drzFilename = Simgen::Util::drzFilename($drz_count,4);
	    my $drzFilename = Simgen::Util::rootFilename($drz_count,4,"drizzle",$getPrefix);
	    if ( $what eq "run" && $doBoolean && $algo eq "drizzle" ) {
		if ( $pixel_scale == $pixel_scale_arg &&
		     $stellar == $stellar_arg ) {
		    print $drzFilename,"\n";
		    exit 0;
		} # End if parameters match input.
	    } # End if.
	    for my $dither ( @dither ) {
		my $rootFilename = Simgen::Util::rootFilename($run_count,4,$algo,$getPrefix);
		if ( $what eq "param" ) {
		    if ( $rootFilename eq $run_arg ) {
			print "stellar = \t$stellar\n";
			print "pixel_scale = \t$pixel_scale\n";
			print "gamma = \tNA\n";
			print "dither = \t@$dither\n";
			exit 0;
		    } # End if root filename matches.
		} elsif ( $what eq "run" && $doBoolean && ( $algo eq "rrg" || $algo eq "shapelets" ) ) {
		    if ( $pixel_scale == $pixel_scale_arg &&
			 $$dither[0] == $dither_arg[0] &&
			 $$dither[1] == $dither_arg[1] &&
			 $stellar == $stellar_arg ) {
			print $rootFilename,"\n";
			exit 0;
		    } # End if parameters match input.
		} # End if.
		$run_count++;
	    } # End for each dither.
	    $drz_count++;
	    $stellar = 0;
	} # End if stellar.
	# Galaxy field.
	my $drzFilename = Simgen::Util::rootFilename($drz_count,4,"drizzle",$getPrefix);
	if ( $what eq "run" && $doBoolean && $algo eq "drizzle" ) {
	    if ( $pixel_scale == $pixel_scale_arg &&
		 $$gamma[0] == $gamma_arg[0] &&
		 $$gamma[1] == $gamma_arg[1] &&
		 $stellar == $stellar_arg ) {
		print $drzFilename,"\n";
		exit 0;
	    } # End if parameters match input.
	} # End if.
	for my $dither ( @dither ) {
	    my $rootFilename = Simgen::Util::rootFilename($run_count,4,$algo,$getPrefix);
	    if ( $what eq "param" ) {
		if ( $rootFilename eq $run_arg ) {
		    print "stellar = \t$stellar\n";
		    print "pixel_scale = \t$pixel_scale\n";
		    print "gamma = \t@$gamma\n";
		    print "dither = \t@$dither\n";
		    exit 0;
		} # End if root filename matches.
	    } elsif ( $what eq "run" && $doBoolean && ( $algo eq "rrg" || $algo eq "shapelets" ) ) {
		if ( $pixel_scale == $pixel_scale_arg &&
		     $$gamma[0] == $gamma_arg[0] &&
		     $$gamma[1] == $gamma_arg[1] &&
		     $$dither[0] == $dither_arg[0] &&
		     $$dither[1] == $dither_arg[1] &&
		     $stellar == $stellar_arg ) {
		    print $rootFilename,"\n";
		    exit 0;
		} # End if parameters match input. 
	    } # End if.
	    $run_count++;
	} # End for each dither.
	$drz_count++;
    } # End for each gamma.
} # End for each pixel scale.

if ( $getTotal ) {
    my $success = 0;
    if ( $algo eq "rrg" || $algo eq "shapelets" ) {
	print "$run_count\n";
    } elsif ( $algo eq "drizzle" ) {
	print "$drz_count\n";
    } else {
	die "ERROR\n";
    }
} else {
    die "ERROR\n";
}
