#!/usr/bin/perl


use strict;
die "Please set the SIMGEN environment variable" unless $ENV{SIMGEN};
use lib "$ENV{SIMGEN}";
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::Locale;
use Simgen::Init;

die "Wrong number of arguments" unless $#ARGV == 1;

my $schemeName = shift;
my $algo = shift;

my $licensesAvail = Simgen::Locale::Licenses("total");
my $idl_dir = Simgen::Locale::IDLdir;

my %t = Simgen::Init::Tweaks();
my $scheme = Simgen::Global::Scheme( $schemeName );
my %scheme = %$scheme;
my $dir = $t{data_home} . "/out/" . $schemeName;
$dir = "$dir/$scheme{rrg_dir}" if $algo eq "rrg";
my $prefix = `simgen_get prefix $algo $schemeName`;
chomp($prefix);

while (1) {

    #system "clear";
    if ( int `wrong_lic_svr` ) {
	system "simkill.pl";
	die "Using wrong idl license server!";
    }
    print "On " . `date`;

    my $n_6  = int `$idl_dir/bin/lmstat -A | grep  '6 licenses' | wc -l`;
    my $n_10 = int `$idl_dir/bin/lmstat -A | grep '10 licenses' | wc -l`;
    my $inUse = 10*$n_10 + 6*$n_6;
    $n_6  = int `$idl_dir/bin/lmstat -A | grep  '6 licenses' | grep fwh | wc -l`;
    $n_10 = int `$idl_dir/bin/lmstat -A | grep '10 licenses' | grep fwh | wc -l`;
    my $inUseByMe = 10*$n_10 + 6*$n_6;
    my $nLicenses = $n_6 + $n_10;

    print "IDL\n";
    print "\tavailable: $licensesAvail  in use: $inUse   in use by me: $inUseByMe\n";
    print "\tinstances: $nLicenses\n";

    print "LOGS\n";
    print "\tfiles: $dir/$prefix*.o.log\n";
    my $nHalted = int `grep alted $dir/$prefix*.o.log | wc -l`;
    my $nWarning = int `grep WARN $dir/$prefix*.o.log | wc -l`;
    $nWarning = $nWarning + int `grep arn $dir/$prefix*.o.log | wc -l`;
    my $nError = int `grep rror $dir/$prefix*.o.log | wc -l`;
    $nError = $nError + int `grep RROR $dir/$prefix*.o.log | wc -l`;
    my $nBenignError = int `grep rror $dir/$prefix*.o.log | grep "arithmetic error" | wc -l`;
    $nError = $nError - $nBenignError;

    print "\thalted: $nHalted   warned: $nWarning   errors: $nError\n";

    my $jobsCmd = "bjobs;echo;bqueues";
    $jobsCmd = "qstat_lite" if ( Simgen::Locale::Cluster eq "SGE" );
#    system "$jobsCmd" if ( Simgen::Global::Institute eq "PDSF" );

    my $nRunning = int `qstat_lite | grep fwh | grep ' r ' | wc -l`;
    my $nWaiting = int `qstat_lite | grep fwh | grep ' qw ' | wc -l`;
    my $nError = int `qstat_lite | grep fwh | grep ' Eqw ' | wc -l`;
    my $nJobs = int `qstat_lite | grep ' fwh ' | wc -l`;
    print "QUEUE\n";
    print "\tupdated: " . `qstat_lite | head -n 1`;
    print "\ttotal: $nJobs   running: $nRunning   waiting: $nWaiting   errors: $nError\n";

    my $nDone = int `grep stoptime $dir/$prefix*.o.log | wc -l`;
    my $total = int `simgen_get total $algo $schemeName`;
    my $nToGo = $total-$nDone;
    print "PROGRESS\n";
    print "\tdone: $nDone   to go: $nToGo   total: $total\n";

    print "___________\n";

    sleep 100;
}
