#!/usr/bin/perl

use strict;
use warnings;
die "Please set the SIMGEN environment variable" unless $ENV{SIMGEN};
use lib "$ENV{SIMGEN}";
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::IO;
use Simgen::Init;
use Simgen::Util;
use Simgen::Engine;

# Parse commandline.
die "Please tell me what you want to compile" if $#ARGV == -1;
my %compile = (
	       simage => false,
	       rrg => false,
	       drizzle => false,
	       );
my $success = false;
foreach my $argv ( @ARGV ) {
    $compile{$argv} = $success = true if $compile{$argv} eq false;
}
die "Bad args @{ARGV}. Please give me a valid algorithm." unless $success;
my $schemeName = undef;
$success = false;
foreach my $argv ( @ARGV ) {
    my $thisScheme = Simgen::Global::Scheme( $argv );
    $success = true, $schemeName = $argv unless $thisScheme eq "ERROR";
}
die "Bad args @{ARGV}. Please give me a valid simulation scheme." unless $success;

# Initialize variables from global libraries.
my %t = Simgen::Init::Tweaks();
my %paramFiles = Simgen::Init::ParamFiles( \%t );
my %software = Simgen::Init::Software( \%t );
my @globalTick = Simgen::Util::tick("simgen global stopwatch");
my $scheme = Simgen::Global::Scheme( $schemeName );
my %scheme = %$scheme;
my @pixel_scale = @{$scheme{pixel_scale}};
my @n_pixels = Simgen::Init::N_Pixels( \@pixel_scale );
my @gamma = @{$scheme{gamma}};
my @dither = @{$scheme{dither}};
$t{psf_file} = $scheme{psf_file};
my $imdir = $t{data_home} . "/out/" . $schemeName;

# Compile.
if ( $compile{simage} ) {
# Compile simage into runtime scripts.
    message("Compiling simage");
    my $count = 0;
    my $pxCount = 0;
    for my $pixel_scale ( @pixel_scale ) {

	my $needStellar = true;

	for my $gamma ( @gamma ) {

	    $t{pixel_scale} = $pixel_scale;
	    $t{n_pixels} = $n_pixels[ $pxCount ];
	    $t{random_seed} = $count + 1;
	    $t{data_in_home} = $t{data_home} . "/in";
	    $t{data_out_home} = $imdir;
	    unless ( mkdir $t{data_out_home} ) {
		if ( -d $t{data_out_home} ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}.");
		    die;
		}
	    }
	    unless ( mkdir "$t{data_out_home}/old" ) {
		if ( -d "$t{data_out_home}/old" ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}/old.");
		    die;
		}
	    }
	    
	    if ( $needStellar ) { 
		for my $dither ( @dither ) {

		    $t{x_dither} = $dither->[0];
		    $t{y_dither} = $dither->[1];
		    $t{input_gamma_1} = 0;
		    $t{input_gamma_2} = 0;
		    $t{n_star} = 372;
		    $t{n_gal} = 0;
		    $t{stellar} = true;
		    $t{noise_seed} = $count + 1;

		    $t{'run'} = `simget.pl run shapelets $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{x_dither} $t{y_dither}`;
		    chomp($t{'run'});
		    die if $t{'run'} eq "ERROR";
		    $t{output_file} = $t{'run'};
		    Simgen::IO::MakeRTScript( "shapelets", "simage", \%software, \%paramFiles, \%t, $t{'run'} );
		    $count++;
		} # End for each dither.

		$needStellar = false;
	    }

	    for my $dither ( @dither ) {
		$t{x_dither} = $dither->[0];
		$t{y_dither} = $dither->[1];
		$t{input_gamma_1} = $gamma->[0];
		$t{input_gamma_2} = $gamma->[1];
		$t{n_star} = 0;
		$t{n_gal} = 6000;
		$t{stellar} = false;
		$t{noise_seed} = $count + 1;

		$t{'run'} = `simget.pl run shapelets $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{x_dither} $t{y_dither}`;
		chomp($t{'run'});
		die if $t{'run'} eq "ERROR";
		$t{output_file} = $t{'run'};
		Simgen::IO::MakeRTScript( "shapelets", "simage", \%software, \%paramFiles, \%t, $t{'run'} );
		
		$count++;
	    } # End for each dither.
	} # End for each gamma.
	$pxCount++;
    } # End for each pixel scale.
    
# Make IDL runtime binaries
    Simgen::Engine::MakeRT( "shapelets", "simage", \%software );
} # End if compile simage.

if ( $compile{rrg} ) {
# Compile RRG into runtime scripts.
    message("Compiling rrg");
    my $count = 0;
    my $pxCount = 0;
    for my $pixel_scale ( @pixel_scale ) {

	my $needStellar = true;
	$t{star_run} = "0";

	for my $gamma ( @gamma ) {

	    $t{pixel_scale} = $pixel_scale;
	    $t{n_pixels} = $n_pixels[ $pxCount ];
	    $t{random_seed} = $count + 1;
	    $t{data_in_home} = $imdir;
	    $t{data_out_home} = "$t{data_in_home}/$scheme{rrg_dir}";
	    unless ( mkdir $t{data_out_home} ) {
		if ( -d $t{data_out_home} ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}.");
		    die;
		}
	    }
	    unless ( mkdir "$t{data_out_home}/old" ) {
		if ( -d "$t{data_out_home}/old" ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}/old.");
		    die;
		}
	    }
	    
	    if ( $needStellar ) {
		for my $dither ( @dither ) {

		    $t{x_dither} = $dither->[0];
		    $t{y_dither} = $dither->[1];
		    $t{input_gamma_1} = 0;
		    $t{input_gamma_2} = 0;
		    $t{n_star} = 372;
		    $t{n_gal} = 0;
		    $t{stellar} = true;
		    $t{noise_seed} = $count + 1;
		    
		    $t{'run'} = `simget.pl run rrg $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{x_dither} $t{y_dither}`;
		    chomp($t{'run'});
		    die if $t{'run'} eq "ERROR";
		    $t{output_file} = $t{'run'};
		    Simgen::IO::MakeRTScript( "rrg", "psf", \%software, \%paramFiles, \%t, $t{'run'} );
		    
		    $count++;
		} # End for each dither.
		$needStellar = false;
	    }
	    
	    for my $dither ( @dither ) {

		$t{x_dither} = $dither->[0];
		$t{y_dither} = $dither->[1];
		$t{input_gamma_1} = $gamma->[0];
		$t{input_gamma_2} = $gamma->[1];
		$t{n_star} = 0;
		$t{n_gal} = 6000;
		$t{stellar} = false;
		$t{noise_seed} = $count + 1;
		
		$t{'run'} = `simget.pl run rrg $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{x_dither} $t{y_dither}`;
		chomp($t{'run'});
		die if $t{'run'} eq "ERROR";
		$t{output_file} = $t{'run'};
		$t{star_run} = `simget.pl run shapelets $schemeName $t{output_file} 1 $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{x_dither} $t{y_dither}`;
		Simgen::IO::MakeRTScript( "rrg", "shear", \%software, \%paramFiles, \%t, $t{'run'} );
		
		$count++;
	    } # End for each dither.
	} # End for each gamma.
	$pxCount++;
    } # End for each pixel scale.
    
    Simgen::Engine::MakeRT( "rrg", "psf", \%software );
    Simgen::Engine::MakeRT( "rrg", "shear", \%software );
} # End if compile rrg.

if ( $compile{drizzle} ) {
    message "Compiling drizzle";

    $t{drz_dir} = $software{drizzle}{hook}{home};

    my $drz_count = 0;
    my $pxCount = 0;
    for my $pixel_scale ( @pixel_scale ) {

	my $needStellar = true;

	for my $gamma ( @gamma ) {

	    $t{pixel_scale} = $pixel_scale;
	    $t{n_pixels} = $n_pixels[ $pxCount ];
	    $t{random_seed} = $drz_count + 1;
	    $t{data_in_home} = $imdir;
	    $t{data_out_home} = $imdir;
	    unless ( mkdir $t{data_out_home} ) {
		if ( -d $t{data_out_home} ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}.");
		    die;
		}
	    }
	    unless ( mkdir "$t{data_out_home}/old" ) {
		if ( -d "$t{data_out_home}/old" ) {
		    #message("Directory $t{data_out_home} already exists.");
		} else {
		    message("Error making output directory $t{data_out_home}/old.");
		    die;
		}
	    }
	    
	    if ( $needStellar ) { 

		$t{input_gamma_1} = 0;
		$t{input_gamma_2} = 0;
		$t{stellar} = true;
		$t{dither_scale} = 0.8;
		
		my $dither_index = 0;
		for my $dither ( @dither ) {

		    my $xsh_string = "dither_xsh$dither_index";
		    my $ysh_string = "dither_ysh$dither_index";
		    $t{$xsh_string}=$dither[$dither_index]->[0];
		    $t{$ysh_string}=$dither[$dither_index]->[1];
		    $t{"dither_fin$dither_index"} = `simget.pl run shapelets $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{$xsh_string} $t{$ysh_string}`;
		    chomp($t{"dither_fin$dither_index"});
		    die if $t{"dither_fin$dither_index"} eq "ERROR";

		    $dither_index++;

		} # End for each dither.

		$t{dither_fout} = `simget.pl run drizzle $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2}`;
		chomp($t{dither_fout});
		die if $t{dither_fout} eq "ERROR";

		Simgen::IO::MakeRTScript( "drizzle", "hook", \%software, \%paramFiles, \%t, $t{dither_fout} );

		$needStellar = false;
		$drz_count++;
	    }

	    $t{input_gamma_1} = $gamma->[0];
	    $t{input_gamma_2} = $gamma->[1];
	    $t{stellar} = true;
	    $t{dither_scale} = 0.8;
	    
	    my $dither_index = 0;
	    for my $dither ( @dither ) {

		my $xsh_string = "dither_xsh$dither_index";
		my $ysh_string = "dither_ysh$dither_index";
		$t{$xsh_string}=$dither[$dither_index]->[0];
		$t{$ysh_string}=$dither[$dither_index]->[1];
		$t{"dither_fin$dither_index"} = `simget.pl run shapelets $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2} $t{$xsh_string} $t{$ysh_string}`;
		chomp($t{"dither_fin$dither_index"});
		die if $t{"dither_fin$dither_index"} eq "ERROR";
		
		$dither_index++;
		
	    } # End for each dither.

	    $t{dither_fout} = `simget.pl run drizzle $schemeName $t{output_file} $needStellar $t{pixel_scale} $t{input_gamma_1} $t{input_gamma_2}`;
	    chomp($t{dither_fout});
	    die if $t{dither_fout} eq "ERROR";
	    
	    Simgen::IO::MakeRTScript( "drizzle", "hook", \%software, \%paramFiles, \%t, $t{dither_fout} );
	    
	    $drz_count++;
	} # End for each gamma.
	$pxCount++;
    } # End for each pixel scale.
    
# Make IDL runtime binaries.
    Simgen::Engine::MakeRT( "drizzle", "hook", \%software );
} # End if compile drizzle.

Simgen::Util::tock(@globalTick);
