#!/usr/bin/perl

use strict;
use warnings;
die "Please set the SIMGEN environment variable" unless $ENV{SIMGEN};
use lib "$ENV{SIMGEN}";
use Simgen::SuperGlobal;
use Simgen::Global;
use Simgen::Locale;
use Simgen::Init;
use Simgen::Util;

my %submit = (
	      "shapelets" => { "simage" => false },
	      "rrg" => { 
		  "psf" => false,
		  "shear" => false
		  },
	      "drizzle" => {
		  "hook" => false,
	      },
	      );
my $success = false;
my $schemeName = $ARGV[0];
my $algo = $ARGV[1];
my $algoMethod = $ARGV[2];
$submit{$algo}{$algoMethod} = $success = true if $submit{$algo}{$algoMethod} eq false;
die "Usage: simsub.pl scheme algo routine stars? gals?\nOptions for 'algo routine':\n\tshapelets simage\n\trrg psf\n\trrg shear\n" unless $success;
foreach my $algos ( keys %submit ) {
    foreach my $algoMethods ( keys %{$submit{$algos}} ) {
	print "algo $algos routine $algoMethods = $submit{$algos}{$algoMethods}\n";
    }
}
{
    my $thisScheme = Simgen::Global::Scheme( $schemeName );
    die "Give a valid scheme name" if $thisScheme eq "ERROR";
}

my $stars = false;
my $gals = false;
$stars = true if $ARGV[3];
$gals = true if $ARGV[4];
$stars = true, $gals = false if $submit{rrg}{psf};
$stars = false, $gals = true if $submit{rrg}{shear};
print "Doing $algo $algoMethod";
print " for stars," if $stars;
print " for gals" if $gals;
print "\n";

my %t = Simgen::Init::Tweaks();
my %software = Simgen::Init::Software( \%t );
my $scriptDir = "";

my $pid;
my $index = 0;
my @pid;
if ( Simgen::Global::Institute eq "Caltech" ) {
    
    die "Not yet ready";
    my @pixel_scale = (
		       0.04,
		       0.05
		       );
    for my $pixel_scale ( @pixel_scale ) {

	unless ($pid[$index] = fork) {
	    my $no;
	  SWITCH: {
	      $no = 1, last SWITCH if ( $pixel_scale == 0.04 );  
	      $no = 2, last SWITCH if ( $pixel_scale == 0.05 );  
	      $no = 3, last SWITCH if ( $pixel_scale == 0.06 );  
	      $no = 4, last SWITCH if ( $pixel_scale == 0.07 );  
	      $no = 5, last SWITCH if ( $pixel_scale == 0.08 );  
	      $no = 6, last SWITCH if ( $pixel_scale == 0.09 );  
	      $no = 7, last SWITCH if ( $pixel_scale == 0.10 );  
	      $no = 1, last SWITCH if ( $pixel_scale == 0.11 );  
	      $no = 2, last SWITCH if ( $pixel_scale == 0.12 );  
	      $no = 3, last SWITCH if ( $pixel_scale == 0.13 );  
	      $no = 4, last SWITCH if ( $pixel_scale == 0.14 );  
#	      $no = 5, last SWITCH if ( $pixel_scale == 0.15 );  
#	      $no = 6, last SWITCH if ( $pixel_scale == 0.16 );  
	      
	      ; # Do nothing.
	  }

	    my $initDotPros;
	    $initDotPros = 0 if !$index;
	    $initDotPros = 0 if $index;

	    chomp( my $host = `hostname` );
	    my $thisPid = $$;
	    die unless ( -d "logs" );
	    my $log = "$t{DATA_OUT_HOME}/simgen_host-${host}_pid-${thisPid}.log";
	    print "ssh -x kishar$no 'hostname; cd simgen; simgen2.pl $initDotPros $pixel_scale' 2>&1 > $log\n";
	    system "ssh -x kishar$no 'hostname; cd simgen; simgen2.pl $initDotPros $pixel_scale' 2>&1 > $log";
	    system "mv $log logs";
	    
	    exit 0; # the child process.
	}
    }
} elsif ( Simgen::Global::Institute eq "PDSF" ) {
    
    my $qCmd = "bsub";
    $qCmd = "qsub" if (Simgen::Locale::Cluster eq "SGE");
    
    my @msg = `module load SGE2 2>&1` if ( Simgen::Locale::Cluster eq "SGE" );
    print @msg;

    my $scheme = Simgen::Global::Scheme( $schemeName );
    my %scheme = %$scheme;
    my @pixel_scale = @{$scheme{pixel_scale}};
    my @gamma = @{$scheme{gamma}};
    my @dither = @{$scheme{dither}};
    $scriptDir = $t{data_home} . "/out/" . $schemeName;
    $scriptDir = "$scriptDir/$scheme{rrg_dir}" if $algo eq "rrg";

    my $starQueue = "short";
    my $galQueue = "medium";
    $galQueue = "short" if $submit{rrg}{shear};
    
    # Do the globular clusters first.
    my $count = 0;
    my $drz_count = 0;
    for my $pixel_scale ( @pixel_scale ) {
	my $stellar = true;
	for my $gamma ( @gamma ) {
	    if ($stellar) {
		if ($stars && $submit{drizzle}{hook}) {
		    my $scriptFilename = `simget.pl run drizzle $schemeName $t{output_file} $stellar $pixel_scale $$gamma[0] $$gamma[1]`;
		    chomp($scriptFilename);
		    die if $scriptFilename eq "ERROR";
		    $scriptFilename = "$scriptDir/${scriptFilename}_$software{$algo}{$algoMethod}{'exec name'}";
		    my $cmd = "$qCmd $scriptFilename";
		    print "$cmd\n";
		    system $cmd;
		    $drz_count++;
		} else {
		    for my $dither ( @dither ) {
			if ($stars) {
			    my $scriptFilename = `simget.pl run $algo $schemeName $t{output_file} $stellar $pixel_scale $$gamma[0] $$gamma[1] $$dither[0] $$dither[1]`;
			    chomp($scriptFilename);
			    die if $scriptFilename eq "ERROR";
			    $scriptFilename = "$scriptDir/${scriptFilename}_$software{$algo}{$algoMethod}{'exec name'}";
			    my $cmd = "$qCmd $scriptFilename";
			    print "$cmd\n";
			    system $cmd;
			}
			$count++;
		    } # End for each dither.
		} # End else.
		$stellar = false;
	    } # End if stellar.
	    if ($submit{drizzle}{hook}) {
		my $scriptFilename = `simget.pl run drizzle $schemeName $t{output_file} $stellar $pixel_scale $$gamma[0] $$gamma[1]`;
		chomp($scriptFilename);
		die if $scriptFilename eq "ERROR";
		$scriptFilename = "$scriptDir/${scriptFilename}_$software{$algo}{$algoMethod}{'exec name'}";
		my $cmd = "$qCmd $scriptFilename";
		print "$cmd\n";
		system $cmd;
		$drz_count++;
	    } else {
		for my $dither ( @dither ) {
		    if ($gals) {
			my $scriptFilename = `simget.pl run $algo $schemeName $t{output_file} $stellar $pixel_scale $$gamma[0] $$gamma[1] $$dither[0] $$dither[1]`;
			chomp($scriptFilename);
			die if $scriptFilename eq "ERROR";
			$scriptFilename = "$scriptDir/${scriptFilename}_$software{$algo}{$algoMethod}{'exec name'}";
			my $cmd = "$qCmd $scriptFilename";
			print "$cmd\n";
			system $cmd;
		    }
		    $count++;
		} # End for each dither.
	    } # End else.
	} # End for each gamma.
    } # End for each pixel scale.
} # End if @ PDSF.

if ( Simgen::Global::Institute eq "Caltech" ) {
    for my $pid ( @pid ){
	waitpid($pid,0);
	print "$pid finished.\n";
    }
}

exec "simonitor.pl $schemeName $algo";
