pro simage_get_ee50,psf,ee50,plot=plot,pixelize=pixelize,op=op,beta=beta,fwhm=fwhm
;get EE50 (the half light radius) for either a shapelet PSF or image
;Davej

if n_params() eq 0 then begin
   print,'-syntax get_ee50,psf,ee50,plot=plot,pixelize=pixelize,op=op,beta=beta,fwhm=fwhm'
   return
endif

s=size(psf)

is_image=0
if s[0] eq 2 then begin
   is_image=1
    ;shapelets_decomp,psf,p
   if n_elements(beta) eq 0 then beta=1.5
    p=simage_shapelets_decomp(psf,beta,12)
    image=psf
endif else begin
    p=psf
endelse

n_max=10.0                      ;number times beta for max r
num=1000                        ;number of points to sample

rmax=p.beta*n_max

r=findgen(num)/(num-1)*rmax

prof=simage_shapelets_profile(p,r)
hmax=0.5*prof[0]
w=where(prof lt hmax)
w=w[0]
r0=r[w-1]
r1=r[w]
dr=r1-r0
;interpolate between these two
m=(prof[w]-prof[w-1])/(r1-r0)
b=prof[w]-m*r1
rhalfmax=(hmax-b)/m                  ;the half max
FWHM=rhalfmax*2.0

I=prof*r

Itrap=(I+shift(I,-1))/2.0
Itrap[num-1]=0.0

trap=1
if trap then begin
    ;trapazoid rule, more accurate
    Cum_light=total(Itrap,/cum)
    Cum_light=shift(Cum_light,1)
    Cum_light[0]=0.0
endif else begin
    ;Just a Riemann sum
    Cum_light=total(I,/cum)
endelse


Cum_light=Cum_light/Cum_light[num-1]


w=where(Cum_light gt 0.5)
w=w[0]

r0=r[w-1]
r1=r[w]
dr=r1-r0
;interpolate between these two
m=(Cum_light[w]-Cum_light[w-1])/(r1-r0)
b=Cum_light[w]-m*r1
rhalf=(0.5-b)/m                  ;the half light radius

EE50=rhalf
EE50_pix=sqrt(0.5/!pi)          ;the half light radius of a pixel

if keyword_set(pixelize) then EE50=sqrt(EE50^2+EE50_pix^2)

if keyword_set(plot) then begin
   
   if is_image then begin
      ss=size(image,/dim)
      nx=ss[0]
      ny=ss[1]
      l=lindgen(nx*ny)
      x=float(l mod nx)-nx/2
      y=float(l/nx)-ny/2
      rad=sqrt(x^2+y^2)
      
   endif
   

    print,'delta r = ',dr
    print,'EE50  = ',ee50
    print,'FWHM = ',FWHM
    pmo=!p.multi
    !p.multi=[0,2,2]
    plot,r,prof,psym=3,tit='Profile',xtit='r (pixels)'
    if is_image and keyword_set(op) then oplot,rad,image[*],color=!red,psym=7
    oplot,r,prof,psym=3
    plot,r,I,tit='Profile * r',psym=3,xtit='r (pixels)'
    if is_image and keyword_set(op) then oplot,rad,image[*]*rad,color=!red,psym=7
    oplot,r,I
    plot,r,Cum_light,tit='Cummulative Light EE!D50!N = '+strcompress(EE50,/rem),psym=3,xtit='r (pixels)'
    oplot,ee50*[1,1],[0,1],color=!red
    oplot,rhalf*[1,1],[0,1],color=!green,linest=2
    plot,r,Cum_light,xr=[r0-5*dr,r1+5*dr],/yno,/xst,psym=-7,tit='Zoom for Interpolation',xtit='r (pixels)'
    oplot,ee50*[1,1],[0,1],color=!red
    oplot,rhalf*[1,1],[0,1],color=!green,linest=2
    oplot,[0,100],[1,1]*0.5,color=!blue
    
    !p.multi=pmo
endif

return
end
