pro simage_read_jpl_psf,file,psf,hdr,fail=fail
;Read the psf ascii files from JPL
;returns info in the hdr (which is ready for output to fits)

if n_params() eq 0 then begin
    print,'-syntax read_jpl_psf,file,psf,hdr,fail=fail'
    return
endif


fail=1                          ;unless it finishes successfully

get_lun,unit
openr,unit,file

line=''

mkhdr,hdr,4,[16,16]
;a simple image hdr
;4 is for floating point data
;will change the size to [512,512] or something after reading 
;the correct values from hdr

p=strpos(file,'Macenka_JPL')
psffile=strmid(file,p+12,strlen(file)-p-12)
ff=strsplit(psffile,'/',/ex)
pp=strcompress(ff[0],/rem)
psfid=strmid(pp,3,1)
ff=ff[1]
ff=strsplit(ff,'.',/ex)
basefile=ff[0]
fxaddpar,hdr,'PSFFILE',psffile
fxaddpar,hdr,'PSF_ID',psfid
fxaddpar,hdr,'BASE',basefile

;Read the header
linenum=0L

maxhead=50                     
;only check this many lines for header info and fail
;if things are not found 

;look for things in this exact order
;if the order changes then just reset linenum each time

head=1
while head and linenum lt maxhead do begin
    readf,unit,line
    if strpos(line,'PSF data:') ne -1 then head=0
    if linenum ge maxhead then begin
        print,'ERROR - can not find "PSF data:" line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile

head=1
while head do begin
    readf,unit,line
    if strpos(line,'Sum of PSF intensity values:') ne -1 then begin
        l=str_sep(line,":")
        if n_elements(l) ne 2 then begin
            print, "ERROR- can not interpret this line"
            print,line
            free_lun,unit
            return
        endif
        sumpsf=float(l[1])
        print,'Sum PSF: ',sumPsf
        fxaddpar,hdr,'SUM_PSF',sumpsf
        head=0
    endif
    if linenum ge maxhead then begin
        print,'ERROR - can not find "Sum of PSF ..." line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile

head=1
while head do begin
    readf,unit,line
    if strpos(line,'Zoom Position') ne -1 and strpos(line,'Field Number') ne -1 then begin
        l=str_sep(line,":")
        if n_elements(l) ne 3 then begin
            print, "ERROR- Can't interpret this line"
            print,line
            free_lun,unit
            return
        endif
    endif
    zoom=str_sep(strtrim(strcompress(l[1]),2),' ')
    zoom=long(zoom[0])
    field=long(l[2])
    print,'zoom: ',zoom
    print,'field: ',field
    fxaddpar,hdr,'ZOOM',zoom
    fxaddpar,hdr,'FIELD',field
    head=0

    if linenum ge maxhead then begin
        print,'ERROR - can not find "Zoom Position ... Field Number" line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile 

head=1
while head do begin
    readf,unit,line
    if strpos(line,'Defocus:') ne -1 then begin
        l=str_sep(line,":")
        if n_elements(l) ne 2 then begin
            print, "ERROR- can not interpret this line"
            free_lun,unit
            print,line
            return
        endif
        ll=str_sep(strtrim(strcompress(l[1]),2),' ')
        defocus=long(ll[0])
        print,'Defocus: ',defocus
        fxaddpar,hdr,'DEFOCUS',defocus
        head=0
    endif
    if linenum ge maxhead then begin
        print,'ERROR - can not find "Defocus:" line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile


head=1
while head do begin
    readf,unit,line
    if strpos(line,'Grid spacing:') ne -1 then begin
        l=str_sep(line,":")
        if n_elements(l) ne 2 then begin
            print, "ERROR- can not interpret this line"
            print,line
            free_lun,unit
            return
        endif
        ll=str_sep(strtrim(strcompress(l[1]),2),' ')
        grid=float(ll[0])
        print,'Grid Spacing: ',grid
        fxaddpar,hdr,'GRID',grid
        head=0
    endif
    if linenum ge maxhead then begin
        print,'ERROR - can not find "Grid spacing:" line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile

head=1
while head do begin
    readf,unit,line
    if strpos(line,'Array Size:') ne -1 then begin
        l=str_sep(line,":")
        if n_elements(l) ne 2 then begin
            print, "ERROR- can not interpret this line"
            print,line
            free_lun,unit
            return
        endif
        nx=long(l[1])
        print,'Array Size: ',nx
        fxaddpar,hdr,'NAXIS1',nx
        fxaddpar,hdr,'NAXIS2',nx
        head=0
    endif
    if linenum ge maxhead then begin
        print,'ERROR - can not find "Sum of PSF ..." line'
        free_lun,unit
        return
    endif
    linenum=linenum+1L
endwhile

psf=replicate(-995.0,nx,nx)

line=''
row=0L
head=1
while head do begin
    readf,unit,line
    line=strtrim(strcompress(line),2)
    l=strsplit(line,' ',/extract)
    Num=n_elements(l)
    if num eq nx then head=0
endwhile

;done with header , now reading data
psf[*,row]=float(l)

row=1L
scale='linear'
l=replicate(-999.0,nx)
while not eof(unit) do begin
    readf,unit,l
    psf[*,row]=float(l)
    if min(l) lt 0 then scale='dB'
    row=row+1L
endwhile

nrows=row
if nrows ne nx then begin
    print,'ERROR - nrows =',nrows,'  not equal to ncols=',nx
    free_lun,unit
    return
endif

;scale to log unis, dB
if scale eq 'linear' then begin
    if min(psf) le 0 then begin
        print,"Can't scale to dB, some pixles are <= 0"
        free_lun,unit
        return
    endif
    psf=alog10(psf)
endif

fxaddpar,hdr,'UNITS','dB'

print,'Original Scaling: ',scale
print

free_lun,unit
fail=0

return
end
